@@ -2,7 +2,7 @@
 #
 # autogen.sh - Generates the initial makefiles from a pristine CVS tree
 #
-# $Id: autogen.sh,v 1.28 2004/07/28 18:12:34 cmdjb Exp $
+# $Id: autogen.sh,v 1.31 2005/02/04 19:53:51 cmdjb Exp $
 #
 # USAGE: autogen.sh [configure options]
 #
@@ -36,7 +36,6 @@ automake_min_vers=1.7
 aclocal_min_vers=$automake_min_vers
 autoconf_min_vers=2.54
 libtoolize_min_vers=1.4
-swig_min_vers=1.3.14
 
 
 automake_args=--add-missing
@@ -59,6 +58,10 @@ automake=automake
 automake_vers=0
 aclocal=aclocal
 aclocal_vers=0
+autoheader=autoheader
+autoheader_vers=0
+libtoolize=libtoolize
+libtoolize_vers=0
 
 here=`pwd`
 while [ $# -ne 0 ] ; do
@@ -109,6 +112,32 @@ while [ $# -ne 0 ] ; do
       fi
     done
   fi
+  progs=`ls libtoolize* 2>/dev/null`
+  if [ "X$progs" != "X" ]; then
+    for prog in $progs; do
+      vers=`$prog --version | sed -ne '1s/^.* //p'`
+      if [ "X$vers" = "X" ]; then
+        continue
+      fi
+      if expr $vers '>' $libtoolize_vers >/dev/null; then
+        libtoolize=$prog
+        libtoolize_vers=$vers
+      fi
+    done
+  fi
+  progs=`ls autoheader* 2>/dev/null`
+  if [ "X$progs" != "X" ]; then
+    for prog in $progs; do
+      vers=`$prog --version | sed -ne '1s/^.* //p'`
+      if [ "X$vers" = "X" ]; then
+        continue
+      fi
+      if expr $vers '>' $autoheader_vers >/dev/null; then
+        autoheader=$prog
+        autoheader_vers=$vers
+      fi
+    done
+  fi
 done
 cd $here
 
@@ -119,6 +148,8 @@ set - $save_args
 echo "$program: Found autoconf $autoconf version $autoconf_vers" 1>&2
 echo "$program: Found automake $automake version $automake_vers" 1>&2
 echo "$program: Found aclocal $aclocal version $aclocal_vers" 1>&2
+echo "$program: Found autoheader $autoheader version $autoheader_vers" 1>&2
+echo "$program: Found libtoolize $libtoolize version $libtoolize_vers" 1>&2
 
 
 if ($autoconf --version) < /dev/null > /dev/null 2>&1 ; then
@@ -165,11 +196,11 @@ else
     DIE="yes"
 fi
 
-if (libtoolize --version) < /dev/null > /dev/null 2>&1 ; then
-    if (libtoolize --version | awk 'NR==1 { if( $4 >= '$libtoolize_min_vers') \
+if ($libtoolize --version) < /dev/null > /dev/null 2>&1 ; then
+    if ($libtoolize --version | awk 'NR==1 { if( $4 >= '$libtoolize_min_vers') \
 			       exit 1; exit 0; }');
     then
-       echo "$program: ERROR: \`libtoolize' is too old."
+       echo "$program: ERROR: \`$libtoolize' is too old."
        echo "           (version $libtoolize_min_vers or newer is required)"
        DIE="yes"
     fi
@@ -181,24 +212,6 @@ else
 fi
 
 
-if (swig -help) </dev/null >/dev/null 2>&1; then 
-  swig_version=`swig -version 2>&1 |sed -ne 's/^SWIG Version //p'`
-  swig_version_dec=`echo $swig_version | awk -F. '{printf("%d\n", 10000*$1 + 100*$2 + $3)};'`
-  swig_min_version_dec=`echo $swig_min_vers | awk -F. '{printf("%d\n", 10000*$1 + 100*$2 + $3)};'`
-
-  if test $swig_version_dec -lt $swig_min_version_dec; then
-    echo "$program: ERROR: \`swig' is too old."
-    echo "           (version $swig_min_vers or newer is required)"
-    DIE="yes"
-  fi
-else
-    echo
-    echo "$program: ERROR: You must have \`swig' installed to compile $PACKAGE."
-    echo "           (version $swig_min_vers or newer is required)"
-    DIE="yes"
-fi
-
-
 if test "X$DIE" != X; then
   exit 1
 fi
@@ -245,20 +258,20 @@ do
 	done
       fi
 
-      echo "$program: Running libtoolize --copy --automake"
+      echo "$program: Running $libtoolize --copy --automake"
       $DRYRUN rm -f ltmain.sh libtool
-      $DRYRUN libtoolize --copy --automake
+      $DRYRUN $libtoolize --copy --automake
 
       aclocalinclude="$ACLOCAL_FLAGS"
-      echo "$program: Running aclocal $aclocalinclude"
+      echo "$program: Running $aclocal $aclocalinclude"
       $DRYRUN $aclocal $aclocal_args
       if grep "^AM_CONFIG_HEADER" configure.ac >/dev/null; then
-	echo "$program: Running autoheader"
-	$DRYRUN autoheader
+	echo "$program: Running $autoheader"
+	$DRYRUN $autoheader
       fi
-      echo "$program: Running automake $am_opt"
+      echo "$program: Running $automake $am_opt"
       $DRYRUN $automake $automake_args $am_opt
-      echo "$program: Running autoconf"
+      echo "$program: Running $autoconf"
       $DRYRUN $autoconf $autoconf_args
 
     )
