@@ -2,20 +2,25 @@
 #
 # rss-view.pl - Redland CGI RSS validator and viewer demo
 #
-# $Id: rss-show.pl,v 1.9 2003/12/16 11:32:41 cmdjb Exp $
+# $Id: rss-show.pl,v 1.10 2004/11/01 20:43:32 cmdjb Exp $
 #
-# Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
-# Institute for Learning and Research Technology - http://www.ilrt.org/
-# University of Bristol - http://www.bristol.ac.uk/
+# Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+# Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+# University of Bristol, UK http://www.bristol.ac.uk/
 # 
-# This package is Free Software or Open Source available under the
-# following licenses (these are alternatives):
-#   1. GNU Lesser General Public License (LGPL)
-#   2. GNU General Public License (GPL)
-#   3. Mozilla Public License (MPL)
+# This package is Free Software and part of Redland http://librdf.org/
+# 
+# It is licensed under the following three licenses as alternatives:
+#   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+#   2. GNU General Public License (GPL) V2 or any newer version
+#   3. Apache License, V2.0 or any newer version
+# 
+# You may not use this file except in compliance with at least one of
+# the above three licenses.
 # 
 # See LICENSE.html or LICENSE.txt at the top of this package for the
-# full license terms.
+# complete terms and further detail along with the license texts for
+# the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
 # 
 # 
 #
@@ -118,15 +123,15 @@ print <<"EOT";
 <h2>About the validator</h2>
 
 <p>This was written using
-<a href="http://www.redland.opensource.ac.uk/">Redland</a>
+<a href="http://librdf.org/">Redland</a>
 and the
-<a href="http://www.redland.opensource.ac.uk/docs/pod/RDF/Redland/RSS.html">RDF::Redland::RSS</a> 
-<a href="http://www.redland.opensource.ac.uk/docs/perl.html">Perl</a> interface,
+<a href="http://librdf.org/docs/pod/RDF/Redland/RSS.html">RDF::Redland::RSS</a> 
+<a href="http://librdf.org/docs/perl.html">Perl</a> interface,
 interpreting the RSS 1.0 content as RDF using the
-<a href="http://www.redland.opensource.ac.uk/raptor/">Raptor RDF parser</a>.</p>
+<a href="http://librdf.org/raptor/">Raptor RDF parser</a>.</p>
 <p>The source code of this demonstration is available in the Redland
 distribution as <tt>demos/rss-show.pl</tt> or from the
-<a href="http://www.redland.opensource.ac.uk/">Redland</a> website</p>
+<a href="http://librdf.org/">Redland</a> website</p>
 
 EOT
 
