@@ -2,9 +2,9 @@
  *
  * redland.h - Redland RDF Application Framework public API
  *
- * $Id: librdf.h,v 1.48 2004/11/01 20:43:32 cmdjb Exp $
+ * $Id: librdf.h,v 1.51 2005/01/30 23:15:06 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -39,7 +39,7 @@ extern "C" {
 #  ifdef REDLAND_STATIC
 #    define REDLAND_API
 #  else
-#    ifdef REDLAND_INTERNAL
+#    ifdef LIBRDF_INTERNAL
 #      define REDLAND_API _declspec(dllexport)
 #    else
 #      define REDLAND_API _declspec(dllimport)
@@ -113,6 +113,8 @@ extern const unsigned int librdf_version_decimal;
 /* FIXME: Should be replaced with automatically pulled
  * definitions from the listed rdf_*.h header files.
  */
+ 
+#ifndef LIBRDF_OBJC_FRAMEWORK
 #include <rdf_log.h>
 #include <rdf_init.h>
 #include <rdf_iterator.h>
@@ -126,7 +128,21 @@ extern const unsigned int librdf_version_decimal;
 #include <rdf_serializer.h>
 #include <rdf_stream.h>
 #include <rdf_query.h>
-
+#else
+#include <Redland/rdf_log.h>
+#include <Redland/rdf_init.h>
+#include <Redland/rdf_iterator.h>
+#include <Redland/rdf_uri.h>
+#include <Redland/rdf_node.h>
+#include <Redland/rdf_concepts.h>
+#include <Redland/rdf_statement.h>
+#include <Redland/rdf_model.h>
+#include <Redland/rdf_storage.h>
+#include <Redland/rdf_parser.h>
+#include <Redland/rdf_serializer.h>
+#include <Redland/rdf_stream.h>
+#include <Redland/rdf_query.h>
+#endif
 
 #ifdef __cplusplus
 }
