@@ -2,25 +2,36 @@
  *
  * rdf_hash_bdb.c - RDF hash Berkeley DB Interface Implementation
  *
- * $Id: rdf_hash_bdb.c,v 1.53 2004/01/30 11:52:18 cmdjb Exp $
+ * $Id: rdf_hash_bdb.c,v 1.58 2004/11/01 20:43:33 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -38,7 +49,7 @@
 #include <db.h>
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 #include <rdf_hash.h>
 #include <rdf_hash_bdb.h>
 
@@ -185,8 +196,8 @@ librdf_hash_bdb_open(void* context, char *identifier,
  *              const char *database, DBTYPE type, u_int32_t flags, int mode);
  */
   if((ret=bdb->open(bdb, file, NULL, DB_BTREE, flags, mode))) {
-    librdf_error(bdb_context->hash->world, "BDB V4.0+ open of '%s' failed - %s",
-                 file, db_strerror(ret));
+    librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "BDB V4.0+ open of '%s' failed - %s", file, db_strerror(ret));
     LIBRDF_FREE(cstring, file);
     return 1;
   }
@@ -198,8 +209,8 @@ librdf_hash_bdb_open(void* context, char *identifier,
  *              const char *database, DBTYPE type, u_int32_t flags, int mode);
  */
   if((ret=bdb->open(bdb, NULL, file, NULL, DB_BTREE, flags, mode))) {
-    librdf_error(bdb_context->hash->world, "BDB V4.1+ open of '%s' failed - %s",
-                 file, db_strerror(ret));
+    librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "BDB V4.1+ open of '%s' failed - %s", file, db_strerror(ret));
     LIBRDF_FREE(cstring, file);
     return 1;
   }
@@ -220,8 +231,8 @@ librdf_hash_bdb_open(void* context, char *identifier,
     flags |= DB_TRUNCATE;
 
   if((ret=db_open(file, DB_BTREE, flags, mode, NULL, &bdb_info, &bdb))) {
-    librdf_error(bdb_context->hash->world, "BDB V2 open of '%s' failed - %d", 
-                 file, ret);
+    librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "BDB V2 open of '%s' failed - %d", file, ret);
     LIBRDF_FREE(cstring, file);
     return 1;
   }
@@ -239,8 +250,8 @@ librdf_hash_bdb_open(void* context, char *identifier,
     remove(file);
 
   if((bdb=dbopen(file, flags, mode, DB_BTREE, NULL)) == 0) {
-    librdf_error(bdb_context->hash->world, "BDB V1 open of '%s' failed - %d",
-                 file, ret);
+    librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "BDB V1 open of '%s' failed - %d", file, ret);
     LIBRDF_FREE(cstring, file);
     return 1;
   }
@@ -547,7 +558,9 @@ librdf_hash_bdb_cursor_get(void* context,
       break;
       
     default:
-      LIBRDF_ERROR2(cursor->hash->hash->world, "Unknown hash method flag %d\n", flags);
+      librdf_log(cursor->hash->hash->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                 "Unknown hash method flag %d\n", flags);
       return 1;
   }
 
