@@ -2,26 +2,37 @@
  *
  * rdf_hash_memory.c - RDF Hash In Memory Implementation
  *
- * $Id: rdf_hash_memory.c,v 1.36 2004/01/14 18:10:32 cmdjb Exp $
+ * $Id: rdf_hash_memory.c,v 1.40 2004/11/01 20:43:33 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -32,7 +43,7 @@
 #include <stdlib.h> /* for abort() as used in errors */
 #endif
 
-#include <librdf.h>
+#include <redland.h>
 #include <rdf_hash_memory.h>
 #include <rdf_types.h>
 
@@ -550,7 +561,9 @@ librdf_hash_memory_cursor_get(void* context,
       
       break;
     default:
-      LIBRDF_ERROR2(cursor->hash->hash->world, "Unknown hash method flag %d\n", flags);
+      librdf_log(cursor->hash->hash->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                 "Unknown hash method flag %d\n", flags);
       return 1;
   }
   
@@ -616,7 +629,9 @@ librdf_hash_memory_cursor_get(void* context,
       
       break;
     default:
-      LIBRDF_ERROR2(cursor->hash->hash->world, "Unknown hash method flag %d\n", flags);
+      librdf_log(cursor->hash->hash->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
+                 "Unknown hash method flag %d\n", flags);
       return 1;
   }
   
