@@ -2,9 +2,9 @@
  *
  * rdf_model.c - RDF Model implementation
  *
- * $Id: rdf_model.c,v 1.97 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_model.c,v 1.101 2005/01/30 23:15:12 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -1242,14 +1242,17 @@ librdf_model_query_execute(librdf_model* model, librdf_query* query)
  * librdf_model_sync - Synchronise the model to the model implementation
  * @model: &librdf_model object
  * 
+ * Return value: non-0 on failure
  **/
-void
+int
 librdf_model_sync(librdf_model* model) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(model, librdf_model);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
 
   if(model->factory->sync)
-    model->factory->sync(model);
+    return model->factory->sync(model);
+
+  return 0;
 }
 
 
@@ -1532,6 +1535,29 @@ librdf_model_to_string(librdf_model* model, librdf_uri *uri,
 }
 
 
+/**
+ * librdf_model_contains_context - Check for a context in the model
+ * @model: the model object
+ * @context: the contest
+ * 
+ * Return value: non 0 if the model contains the context node
+ **/
+int
+librdf_model_contains_context(librdf_model* model, librdf_node* context) {
+  librdf_stream* stream;
+  int result;
+
+  stream=librdf_model_context_as_stream(model, context);
+  if(!stream)
+    return 0;
+  
+  result=!librdf_stream_end(stream);
+  librdf_free_stream(stream);
+
+  return result;
+}
+
+
 #endif
 
 
@@ -1783,6 +1809,16 @@ main(int argc, char *argv[])
     }
   }
 
+#define TEST_CONTEXT_URI_INDEX 0
+
+  if(librdf_model_contains_context(model, nodes[TEST_CONTEXT_URI_INDEX])) {
+    fprintf(stderr, "%s: Model contains context %s\n", program, 
+            librdf_uri_as_string(uris[TEST_CONTEXT_URI_INDEX]));
+  } else {
+    fprintf(stderr, "%s: librdf_model_contains_contexts failed to find context URI %s\n", 
+            program, librdf_uri_as_string(uris[TEST_CONTEXT_URI_INDEX]));
+    exit(1);
+  }
   
 
   for (i=0; i<URI_STRING_COUNT; i++) {
@@ -1802,7 +1838,7 @@ main(int argc, char *argv[])
                                         &string_length);
   if(string_length != EXPECTED_BAD_STRING_LENGTH) {
     fprintf(stderr, "%s: Serialising to RDF/XML returned string size %d, expected %d\n", program,
-            string_length, EXPECTED_BAD_STRING_LENGTH);
+            (int)string_length, EXPECTED_BAD_STRING_LENGTH);
     return 1;
   }
   librdf_free_uri(base_uri);
