@@ -2,9 +2,9 @@
  *
  * rdf_model.h - RDF Model definition
  *
- * $Id: rdf_model.h,v 1.48 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_model.h,v 1.52 2005/01/30 23:15:14 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -30,7 +30,11 @@
 #ifndef LIBRDF_MODEL_H
 #define LIBRDF_MODEL_H
 
+#ifndef LIBRDF_OBJC_FRAMEWORK
 #include <rdf_uri.h>
+#else
+#include <Redland/rdf_uri.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -142,7 +146,7 @@ struct librdf_model_factory_s {
   librdf_query_results* (*query_execute)(librdf_model* model, librdf_query* query);
 
   /* sync the model to the storage - OPTIONAL */
-  void (*sync)(librdf_model* model);
+  int (*sync)(librdf_model* model);
 
   /* add a statement from the context - OPTIONAL (librdf_model will
    * implement using context_add_statement if missing) 
@@ -265,11 +269,12 @@ REDLAND_API int librdf_model_context_remove_statement(librdf_model* model, librd
 REDLAND_API int librdf_model_context_remove_statements(librdf_model* model, librdf_node* context);
 REDLAND_API librdf_stream* librdf_model_context_as_stream(librdf_model* model, librdf_node* context);
 REDLAND_API REDLAND_DEPRECATED librdf_stream* librdf_model_context_serialize(librdf_model* model, librdf_node* context);
+REDLAND_API int librdf_model_contains_context(librdf_model* model, librdf_node* context);
 
 /* query language */
 REDLAND_API librdf_query_results* librdf_model_query_execute(librdf_model* model, librdf_query* query);
 
-REDLAND_API void librdf_model_sync(librdf_model* model);
+REDLAND_API int librdf_model_sync(librdf_model* model);
 
 REDLAND_API librdf_storage* librdf_model_get_storage(librdf_model *model);
 
