@@ -2,7 +2,7 @@
  *
  * rdf_model_storage.c - RDF Model Storage implementation
  *
- * $Id: rdf_model_storage.c,v 1.18 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_model_storage.c,v 1.19 2004/11/10 10:20:34 cmdjb Exp $
  *
  * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -492,13 +492,14 @@ librdf_model_storage_query_execute(librdf_model* model,
 /**
  * librdf_model_storage_sync - Synchronise the model to the storage
  * @model: &librdf_model object
- * 
+ *
+ * Return-value: Non-0 on failure
  **/
-static void
+static int
 librdf_model_storage_sync(librdf_model* model) 
 {
   librdf_model_storage_context *context=(librdf_model_storage_context *)model->context;
-  librdf_storage_sync(context->storage);
+  return librdf_storage_sync(context->storage);
 }
 
 
