@@ -2,7 +2,7 @@
  *
  * rdf_node.h - RDF Node definition
  *
- * $Id: rdf_node.h,v 1.46 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_node.h,v 1.47 2004/11/10 10:27:21 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -30,7 +30,11 @@
 #ifndef LIBRDF_NODE_H
 #define LIBRDF_NODE_H
 
+#ifndef LIBRDF_OBJC_FRAMEWORK
 #include <rdf_uri.h>
+#else
+#include <Redland/rdf_uri.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
