@@ -2,7 +2,7 @@
  *
  * rdf_parser.c - RDF Parser Factory implementation
  *
- * $Id: rdf_parser.c,v 1.57 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_parser.c,v 1.58 2004/12/23 23:12:17 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -192,14 +192,20 @@ librdf_get_parser_factory(librdf_world *world,
 	continue;
 
       /* MIME type may need to match */
-      if(mime_type && factory->mime_type &&
-         strcmp(factory->mime_type, mime_type))
-        continue;
+      if(mime_type) {
+        if(!factory->mime_type)
+          continue;
+        if(strcmp(factory->mime_type, mime_type))
+          continue;
+      }
       
       /* URI may need to match */
-      if(type_uri && factory->type_uri &&
-         librdf_uri_equals(factory->type_uri, type_uri))
-        continue;
+      if(type_uri) {
+        if(!factory->type_uri)
+          continue;
+        if(librdf_uri_equals(factory->type_uri, type_uri))
+          continue;
+      }
 
       /* found it */
       break;
