@@ -2,26 +2,37 @@
  *
  * rdf_parser.c - RDF Parser using Raptor
  *
- * $Id: rdf_parser_raptor.c,v 1.66 2004/01/31 18:49:42 cmdjb Exp $
+ * $Id: rdf_parser_raptor.c,v 1.79 2004/11/01 20:43:34 cmdjb Exp $
  *
- * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -32,10 +43,7 @@
 #include <errno.h>
 #endif
 
-#include <librdf.h>
-
-#define RAPTOR_IN_REDLAND
-#include <raptor.h>
+#include <redland.h>
 
 
 /* serialising implementing functions */
@@ -48,6 +56,7 @@ static void librdf_parser_raptor_serialise_finished(void* context);
 typedef struct {
   librdf_parser *parser;        /* librdf parser object */
   librdf_hash *bnode_hash;      /* bnode id (raptor=>internal) map */
+  raptor_parser *rdf_parser;      /* source URI string (for raptor) */
   char *parser_name;            /* raptor parser name to use */
 
   int errors;
@@ -57,7 +66,6 @@ typedef struct {
 
 typedef struct {
   librdf_parser_raptor_context* pcontext; /* parser context */
-  raptor_parser *rdf_parser;      /* source URI string (for raptor) */
 
   /* when reading from a file */
   FILE *fh;
@@ -102,7 +110,10 @@ librdf_parser_raptor_init(librdf_parser *parser, void *context)
   /* New in-memory hash for mapping bnode IDs */
   pcontext->bnode_hash=librdf_new_hash(parser->world, NULL);
 
-  /* always succeeds ? */  
+  pcontext->rdf_parser=raptor_new_parser(pcontext->parser_name);
+  if(!pcontext->rdf_parser)
+    return 1;
+
   return 0;
 }
   
@@ -115,6 +126,9 @@ static void
 librdf_parser_raptor_terminate(void *context) 
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+  
+  if(pcontext->rdf_parser)
+    raptor_free_parser(pcontext->rdf_parser);
 
   if(pcontext->bnode_hash)
     librdf_free_hash(pcontext->bnode_hash);
@@ -129,8 +143,8 @@ librdf_parser_raptor_terminate(void *context)
  * Adds the statement to the list of statements.
  */
 static void
-librdf_parser_raptor_new_statement_handler (void *context,
-                                            const raptor_statement *rstatement)
+librdf_parser_raptor_new_statement_handler(void *context,
+                                           const raptor_statement *rstatement)
 {
   librdf_parser_raptor_stream_context* scontext=(librdf_parser_raptor_stream_context*)context;
   librdf_node* node;
@@ -154,7 +168,9 @@ librdf_parser_raptor_new_statement_handler (void *context,
                                                     scontext->source_uri,
                                                     scontext->base_uri);
   } else {
-    LIBRDF_ERROR2(world, "Unknown Raptor subject identifier type %d", rstatement->subject_type);
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Unknown Raptor subject identifier type %d", rstatement->subject_type);
     librdf_free_statement(statement);
     return;
   }
@@ -177,7 +193,9 @@ librdf_parser_raptor_new_statement_handler (void *context,
                                                     scontext->source_uri,
                                                     scontext->base_uri);
   } else {
-    LIBRDF_ERROR2(world, "Unknown Raptor predicate identifier type %d", rstatement->predicate_type);
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Unknown Raptor predicate identifier type %d", rstatement->predicate_type);
     librdf_free_statement(statement);
     return;
   }
@@ -190,7 +208,6 @@ librdf_parser_raptor_new_statement_handler (void *context,
     int is_xml_literal = (rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL);
     librdf_uri *datatype_uri=(librdf_uri*)rstatement->object_literal_datatype;
     
-    
     if(is_xml_literal)
       librdf_statement_set_object(statement,
                                  librdf_new_node_from_literal(world,
@@ -214,7 +231,9 @@ librdf_parser_raptor_new_statement_handler (void *context,
                                                     scontext->base_uri);
     librdf_statement_set_object(statement, node);
   } else {
-    LIBRDF_ERROR2(world, "Unknown Raptor object identifier type %d", rstatement->object_type);
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Unknown Raptor object identifier type %d", rstatement->object_type);
     librdf_free_statement(statement);
     return;
   }
@@ -243,29 +262,14 @@ librdf_parser_raptor_error_handler(void *data, raptor_locator *locator,
   librdf_parser_raptor_stream_context* scontext=(librdf_parser_raptor_stream_context*)data;
   librdf_parser_raptor_context* pcontext;
   librdf_parser* parser;
-  static const char *message_prefix=" - Raptor error - ";
-  int prefix_len=strlen(message_prefix);
-  int message_len=strlen(message);
-  int locator_len=raptor_format_locator(NULL, 0, locator);
-  char *buffer;
 
   pcontext=scontext->pcontext;
   parser=pcontext->parser;
 
   pcontext->errors++;
-  raptor_parse_abort(scontext->rdf_parser);
+  raptor_parse_abort(pcontext->rdf_parser);
 
-  buffer=(char*)LIBRDF_MALLOC(cstring, locator_len+prefix_len+message_len+1);
-  if(!buffer) {
-    fprintf(stderr, "librdf_raptor_error_handler: Out of memory\n");
-    return;
-  }
-  raptor_format_locator(buffer, locator_len, locator);
-  strncpy(buffer+locator_len, message_prefix, prefix_len);
-  strcpy(buffer+prefix_len+locator_len, message); /* want extra \0 - using strcpy */
-
-  librdf_error(parser->world, "%s", buffer);
-  LIBRDF_FREE(cstring, buffer);
+  librdf_log_simple(pcontext->parser->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, locator, message);
 }
 
 
@@ -275,29 +279,11 @@ librdf_parser_raptor_warning_handler(void *data, raptor_locator *locator,
 {
   librdf_parser_raptor_stream_context* scontext=(librdf_parser_raptor_stream_context*)data;
   librdf_parser_raptor_context* pcontext;
-  librdf_parser* parser;
-  static const char *message_prefix=" - Raptor warning - ";
-  int prefix_len=strlen(message_prefix);
-  int message_len=strlen(message);
-  int locator_len=raptor_format_locator(NULL, 0, locator);
-  char *buffer;
 
   pcontext=scontext->pcontext;
-  parser=pcontext->parser;
-
   pcontext->warnings++;
 
-  buffer=(char*)LIBRDF_MALLOC(cstring, locator_len+prefix_len+message_len+1);
-  if(!buffer) {
-    fprintf(stderr, "librdf_raptor_warning_handler: Out of memory\n");
-    return;
-  }
-  raptor_format_locator(buffer, locator_len, locator);
-  strncpy(buffer+locator_len, message_prefix, prefix_len);
-  strcpy(buffer+prefix_len+locator_len, message); /* want extra \0 - using strcpy */
-
-  librdf_warning(parser->world, "%s", message);
-  LIBRDF_FREE(cstring, buffer);
+  librdf_log_simple(pcontext->parser->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_PARSER, locator, message);
 }
 
 
@@ -322,7 +308,7 @@ librdf_parser_raptor_get_next_statement(librdf_parser_raptor_stream_context *con
   context->current=NULL;
   while(!feof(context->fh)) {
     int len=fread(buffer, 1, RAPTOR_IO_BUFFER_LEN, context->fh);
-    int ret=raptor_parse_chunk(context->rdf_parser, buffer, len, 
+    int ret=raptor_parse_chunk(context->pcontext->rdf_parser, buffer, len, 
                                (len < RAPTOR_IO_BUFFER_LEN));
     if(ret) {
       status=(-1);
@@ -362,7 +348,7 @@ librdf_parser_raptor_generate_id_handler(void *user_data,
       if(librdf_hash_put_strings(pcontext->bnode_hash, (char*)user_bnodeid, (char*)mapped_id))
         return NULL;
     }
-    SYSTEM_FREE((char*)user_bnodeid);
+    raptor_free_memory(user_bnodeid);
     return mapped_id;
   }
   else
@@ -386,7 +372,6 @@ librdf_parser_raptor_parse_file_as_stream(void *context, librdf_uri *uri,
   librdf_parser_raptor_stream_context* scontext;
   librdf_stream *stream;
   char* filename;
-  raptor_parser *rdf_parser;
   int rc;
   librdf_world *world=uri->world;
 
@@ -397,24 +382,18 @@ librdf_parser_raptor_parse_file_as_stream(void *context, librdf_uri *uri,
   if(!scontext)
     return NULL;
 
-  rdf_parser=raptor_new_parser(pcontext->parser_name);
-  if(!rdf_parser)
-    return NULL;
-
-  raptor_set_statement_handler(rdf_parser, scontext, 
+  raptor_set_statement_handler(pcontext->rdf_parser, scontext, 
                                librdf_parser_raptor_new_statement_handler);
   
-  raptor_set_error_handler(rdf_parser, scontext, 
+  raptor_set_error_handler(pcontext->rdf_parser, scontext, 
                            librdf_parser_raptor_error_handler);
-  raptor_set_warning_handler(rdf_parser, scontext,
+  raptor_set_warning_handler(pcontext->rdf_parser, scontext,
                              librdf_parser_raptor_warning_handler);
 
-  raptor_set_generate_id_handler(rdf_parser, pcontext,
+  raptor_set_generate_id_handler(pcontext->rdf_parser, pcontext,
                                  librdf_parser_raptor_generate_id_handler);
   
   
-  scontext->rdf_parser=rdf_parser;
-
   scontext->pcontext=pcontext;
   scontext->source_uri = librdf_new_uri_from_uri(uri);
   if(!base_uri)
@@ -435,7 +414,7 @@ librdf_parser_raptor_parse_file_as_stream(void *context, librdf_uri *uri,
   SYSTEM_FREE(filename);
 
   /* Start the parse */
-  rc=raptor_start_parse(rdf_parser, (raptor_uri*)base_uri);
+  rc=raptor_start_parse(pcontext->rdf_parser, (raptor_uri*)base_uri);
 
   /* start parsing; initialises scontext->statements, scontext->current */
   librdf_parser_raptor_get_next_statement(scontext);
@@ -465,7 +444,7 @@ librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler(raptor_www *www,
   librdf_parser_raptor_stream_context* scontext=(librdf_parser_raptor_stream_context*)userdata;
   int len=size*nmemb;
 
-  if(raptor_parse_chunk(scontext->rdf_parser, (const unsigned char*)ptr, len, 0))
+  if(raptor_parse_chunk(scontext->pcontext->rdf_parser, (const unsigned char*)ptr, len, 0))
     raptor_www_abort(www, "Parsing failed");
 }
 
@@ -475,6 +454,7 @@ librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler(raptor_www *www,
  * @context: parser context
  * @uri: &librdf_uri URI of ontent source
  * @string: or content string
+ * @length: length of the string or 0 if not yet counted
  * @base_uri: &librdf_uri URI of the content location or NULL if the same
  *
  * Only one of uri or string may be given
@@ -483,11 +463,11 @@ librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler(raptor_www *www,
 static librdf_stream*
 librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
                                             const unsigned char *string,
+                                            size_t length,
                                             librdf_uri *base_uri)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
   librdf_parser_raptor_stream_context* scontext;
-  raptor_parser *rdf_parser;
   librdf_stream *stream;
   
   pcontext->errors=0;
@@ -501,26 +481,18 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   if(!scontext)
     return NULL;
 
-  rdf_parser=raptor_new_parser(pcontext->parser_name);
-  if(!rdf_parser) {
-    LIBRDF_FREE(librdf_parser_raptor_stream_context, scontext);
-    return NULL;
-  }
-
-  raptor_set_statement_handler(rdf_parser, scontext, 
+  raptor_set_statement_handler(pcontext->rdf_parser, scontext, 
                                librdf_parser_raptor_new_statement_handler);
   
-  raptor_set_error_handler(rdf_parser, scontext, 
+  raptor_set_error_handler(pcontext->rdf_parser, scontext, 
                            librdf_parser_raptor_error_handler);
-  raptor_set_warning_handler(rdf_parser, scontext,
+  raptor_set_warning_handler(pcontext->rdf_parser, scontext,
                              librdf_parser_raptor_warning_handler);
 
-  raptor_set_generate_id_handler(rdf_parser, pcontext,
+  raptor_set_generate_id_handler(pcontext->rdf_parser, pcontext,
                                  librdf_parser_raptor_generate_id_handler);
   
   
-  scontext->rdf_parser=rdf_parser;
-
   scontext->pcontext=pcontext;
   if(!base_uri)
     base_uri=uri;
@@ -539,7 +511,6 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
     raptor_www *www=raptor_www_new();
     if(!www) {
       LIBRDF_FREE(librdf_parser_raptor_stream_context, scontext);
-      raptor_free_parser(rdf_parser);
       return NULL;
     }
     
@@ -547,20 +518,22 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
                                        librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler,
                                        scontext);
     
-    if(raptor_start_parse(rdf_parser, (raptor_uri*)base_uri)) {
+    if(raptor_start_parse(pcontext->rdf_parser, (raptor_uri*)base_uri)) {
       raptor_www_free(www);
       return NULL;
     }
     
     raptor_www_fetch(www, (raptor_uri*)uri);
-    raptor_parse_chunk(rdf_parser, NULL, 0, 1);
+    raptor_parse_chunk(pcontext->rdf_parser, NULL, 0, 1);
 
     raptor_www_free(www);
   } else {
-    if(raptor_start_parse(rdf_parser, (raptor_uri*)base_uri))
+    if(raptor_start_parse(pcontext->rdf_parser, (raptor_uri*)base_uri))
       return NULL;
     
-    raptor_parse_chunk(rdf_parser, string, strlen((const char*)string), 1);
+    if(!length)
+      length=strlen((const char*)string);
+    raptor_parse_chunk(pcontext->rdf_parser, string, length, 1);
   }
   
 
@@ -594,8 +567,9 @@ static librdf_stream*
 librdf_parser_raptor_parse_uri_as_stream(void *context, librdf_uri *uri,
                                          librdf_uri *base_uri)
 {
-  return librdf_parser_raptor_parse_as_stream_common(context, 
-                                                     uri, NULL, base_uri);
+  return librdf_parser_raptor_parse_as_stream_common(context, uri,
+                                                     NULL, 0,
+                                                     base_uri);
 }
 
 
@@ -612,19 +586,43 @@ librdf_parser_raptor_parse_string_as_stream(void *context,
                                             const unsigned char *string,
                                             librdf_uri *base_uri)
 {
-  return librdf_parser_raptor_parse_as_stream_common(context, 
-                                                     NULL, string, base_uri);
+  return librdf_parser_raptor_parse_as_stream_common(context, NULL,
+                                                     string, 0,
+                                                     base_uri);
 }
 
 
+/**
+ * librdf_parser_raptor_parse_counted_string_as_stream - Parse a counted string of content to a librdf_stream of statements
+ * @context: parser context
+ * @string: string content to parse
+ * @length: length of the string content (must be >0)
+ * @base_uri: the base URI to use
+ * 
+ * Return value: &librdf_stream of statements or NULL
+ **/
+static librdf_stream*
+librdf_parser_raptor_parse_counted_string_as_stream(void *context,
+                                                    const unsigned char *string,
+                                                    size_t length,
+                                                    librdf_uri* base_uri) 
+{
+  return librdf_parser_raptor_parse_as_stream_common(context, NULL,
+                                                     string, length,
+                                                     base_uri);
+}
+
 /**
  * librdf_parser_raptor_parse_uri_into_model - Retrieve the RDF/XML content at URI and store it into a librdf_model
  * @context: parser context
  * @uri: &librdf_uri URI of RDF/XML content source
+ * @string: string content to parser
+ * @length: length of the string or 0 if not yet counted
  * @base_uri: &librdf_uri URI of the content location
  * @model: &librdf_model of model
  *
- * Retrieves all statements and stores them in the given model.
+ * Parses the content at @uri or @string and store it in the given model.
+ * A base URI must be given if @uri is NULL, when @string is used.
  *
  * Return value: non 0 on failure
  **/
@@ -632,13 +630,13 @@ static int
 librdf_parser_raptor_parse_into_model_common(void *context,
                                              librdf_uri *uri, 
                                              const unsigned char *string,
+                                             size_t length,
                                              librdf_uri *base_uri,
                                              librdf_model* model)
 {
   int status=0;
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
   librdf_parser_raptor_stream_context* scontext;
-  raptor_parser *rdf_parser;
   
   pcontext->errors=0;
   pcontext->warnings=0;
@@ -647,25 +645,18 @@ librdf_parser_raptor_parse_into_model_common(void *context,
   if(!scontext)
     return 1;
 
-  rdf_parser=raptor_new_parser(pcontext->parser_name);
-  if(!rdf_parser) {
-    LIBRDF_FREE(librdf_parser_raptor_stream_context, scontext);
-    return 1;
-  }
-
-  raptor_set_statement_handler(rdf_parser, scontext, 
+  raptor_set_statement_handler(pcontext->rdf_parser, scontext, 
                                librdf_parser_raptor_new_statement_handler);
   
-  raptor_set_error_handler(rdf_parser, scontext, 
+  raptor_set_error_handler(pcontext->rdf_parser, scontext, 
                            librdf_parser_raptor_error_handler);
-  raptor_set_warning_handler(rdf_parser, scontext,
+  raptor_set_warning_handler(pcontext->rdf_parser, scontext,
                              librdf_parser_raptor_warning_handler);
 
-  raptor_set_generate_id_handler(rdf_parser, pcontext,
+  raptor_set_generate_id_handler(pcontext->rdf_parser, pcontext,
                                  librdf_parser_raptor_generate_id_handler);
   
   
-  scontext->rdf_parser=rdf_parser;
   scontext->pcontext=pcontext;
 
   if(!base_uri)
@@ -686,11 +677,15 @@ librdf_parser_raptor_parse_into_model_common(void *context,
   scontext->model = model;
 
   if(uri) {
-    status=raptor_parse_uri(rdf_parser, (raptor_uri*)uri, (raptor_uri*)base_uri);
+    status=raptor_parse_uri(pcontext->rdf_parser, (raptor_uri*)uri, (raptor_uri*)base_uri);
   } else {
-    status=raptor_start_parse(rdf_parser, (raptor_uri*)base_uri);
-    if(!status)
-      status=raptor_parse_chunk(rdf_parser, string, strlen((const char*)string), 1);
+    status=raptor_start_parse(pcontext->rdf_parser, (raptor_uri*)base_uri);
+    if(!status) {
+      if(!length)
+        length=strlen((const char*)string);
+      status=raptor_parse_chunk(pcontext->rdf_parser, string, length, 1);
+    }
+    
   }
   
 
@@ -704,7 +699,7 @@ librdf_parser_raptor_parse_into_model_common(void *context,
  * librdf_parser_raptor_parse_uri_into_model - Retrieve the RDF/XML content at URI and store it into a librdf_model
  * @context: parser context
  * @uri: &librdf_uri URI of RDF/XML content source
- * @base_uri: &librdf_uri URI of the content location
+ * @base_uri: &librdf_uri URI of the content location (or NULL if the same as @uri)
  * @model: &librdf_model of model
  *
  * Retrieves all statements and stores them in the given model.
@@ -716,15 +711,15 @@ librdf_parser_raptor_parse_uri_into_model(void *context, librdf_uri *uri,
                                           librdf_uri *base_uri,
                                           librdf_model* model)
 {
-  return librdf_parser_raptor_parse_into_model_common(context,
-                                                      uri, NULL,
+  return librdf_parser_raptor_parse_into_model_common(context, uri, 
+                                                      NULL, 0,
                                                       base_uri, model);}
 
 
 /**
  * librdf_parser_raptor_parse_string_into_model - Parse the RDF/XML content in a string and store it into a librdf_model
  * @context: parser context
- * @string: RDF/XML source
+ * @string: content to parse
  * @base_uri: &librdf_uri URI of the content location
  * @model: &librdf_model of model
  *
@@ -738,8 +733,31 @@ librdf_parser_raptor_parse_string_into_model(void *context,
                                              librdf_uri *base_uri,
                                              librdf_model* model)
 {
-  return librdf_parser_raptor_parse_into_model_common(context,
-                                                      NULL, string,
+  return librdf_parser_raptor_parse_into_model_common(context, NULL,
+                                                      string, 0,
+                                                      base_uri, model);
+}
+
+
+/**
+ * librdf_parser_raptor_parse_counted_string_into_model - Parse a counted string of content into an librdf_model
+ * @context: parser context
+ * @string: the content to parse
+ * @length: length of content (must be >0)
+ * @base_uri: the base URI to use
+ * @model: the model to use
+ * 
+ * Return value: non 0 on failure
+ **/
+static int
+librdf_parser_raptor_parse_counted_string_into_model(void *context,
+                                                     const unsigned char *string,
+                                                     size_t length,
+                                                     librdf_uri* base_uri, 
+                                                     librdf_model* model)
+{
+  return librdf_parser_raptor_parse_into_model_common(context, NULL,
+                                                      string, length,
                                                       base_uri, model);
 }
 
@@ -793,6 +811,7 @@ librdf_parser_raptor_serialise_next_statement(void* context)
 /**
  * librdf_parser_raptor_serialise_get_statement - Get the current librdf_statement from the stream of statements from the raptor RDF parse
  * @context: the context passed in by &librdf_stream
+ * @flags: the context get method flags
  * 
  * Return value: a new &librdf_statement or NULL on error or if no statements found.
  **/
@@ -809,7 +828,9 @@ librdf_parser_raptor_serialise_get_statement(void* context, int flags)
       return NULL;
       
     default:
-      LIBRDF_ERROR2(scontext->pcontext->parser->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(scontext->pcontext->parser->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       return NULL;
   }
 
@@ -828,9 +849,6 @@ librdf_parser_raptor_serialise_finished(void* context)
   if(scontext) {
     librdf_statement* statement;
 
-    if(scontext->rdf_parser)
-      raptor_free_parser(scontext->rdf_parser);
-
     if(scontext->current)
       librdf_free_statement(scontext->current);
 
@@ -853,6 +871,7 @@ static librdf_node*
 librdf_parser_raptor_get_feature(void* context, librdf_uri *feature) 
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+  static unsigned char intbuffer[20]; /* FIXME */
   unsigned char *uri_string;
 
   if(!feature)
@@ -863,23 +882,53 @@ librdf_parser_raptor_get_feature(void* context, librdf_uri *feature)
     return NULL;
   
   if(!strcmp((const char*)uri_string, LIBRDF_PARSER_FEATURE_ERROR_COUNT)) {
-    static unsigned char count[20]; /* FIXME */
-
-    sprintf((char*)count, "%d", pcontext->errors);
+    sprintf((char*)intbuffer, "%d", pcontext->errors);
     return librdf_new_node_from_typed_literal(pcontext->parser->world,
-                                              count, NULL, NULL);
+                                              intbuffer, NULL, NULL);
   } else if(!strcmp((const char*)uri_string, LIBRDF_PARSER_FEATURE_WARNING_COUNT)) {
-    static unsigned char count[20]; /* FIXME */
-
-    sprintf((char*)count, "%d", pcontext->warnings);
+    sprintf((char*)intbuffer, "%d", pcontext->warnings);
     return librdf_new_node_from_typed_literal(pcontext->parser->world,
-                                              count, NULL, NULL);
+                                              intbuffer, NULL, NULL);
+  } else {
+    /* try a raptor feature */
+    raptor_feature feature_i=raptor_feature_from_uri((raptor_uri*)feature);
+    if((int)feature_i >= 0) {
+      int value=raptor_get_feature(pcontext->rdf_parser, feature_i);
+      sprintf((char*)intbuffer, "%d", value);
+      return librdf_new_node_from_typed_literal(pcontext->parser->world,
+                                                intbuffer, NULL, NULL);
+    }
   }
-
+  
   return NULL;
 }
 
 
+static int
+librdf_parser_raptor_set_feature(void* context, 
+                                 librdf_uri *feature, librdf_node *value) 
+{
+  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+  raptor_feature feature_i;
+  int value_i;
+  
+  if(!feature)
+    return 1;
+
+  /* try a raptor feature */
+  feature_i=raptor_feature_from_uri((raptor_uri*)feature);
+  if((int)feature_i < 0)
+    return 1;
+  
+  if(!librdf_node_is_literal(value))
+    return 1;
+  
+  value_i=atoi((const char*)librdf_node_get_literal_value(value));
+
+  return raptor_set_feature(pcontext->rdf_parser, feature_i, value_i);
+}
+
+
 
 static raptor_uri*
 librdf_raptor_new_uri(void *context, const unsigned char *uri_string) 
@@ -947,6 +996,7 @@ librdf_raptor_uri_as_counted_string(void *context, raptor_uri *uri, size_t *len_
 }
 
 
+
 static raptor_uri_handler librdf_raptor_uri_handler = {
   librdf_raptor_new_uri,
   librdf_raptor_new_uri_from_uri_local_name,
@@ -977,7 +1027,10 @@ librdf_parser_raptor_register_factory(librdf_parser_factory *factory)
   factory->parse_uri_into_model = librdf_parser_raptor_parse_uri_into_model;
   factory->parse_string_as_stream = librdf_parser_raptor_parse_string_as_stream;
   factory->parse_string_into_model = librdf_parser_raptor_parse_string_into_model;
+  factory->parse_counted_string_as_stream = librdf_parser_raptor_parse_counted_string_as_stream;
+  factory->parse_counted_string_into_model = librdf_parser_raptor_parse_counted_string_into_model;
   factory->get_feature = librdf_parser_raptor_get_feature;
+  factory->set_feature = librdf_parser_raptor_set_feature;
 }
 
 
