@@ -2,9 +2,9 @@
  *
  * rdf_query.c - RDF Query Adaptors Implementation
  *
- * $Id: rdf_query.c,v 1.46 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_query.c,v 1.52 2005/01/30 23:15:21 cmdjb Exp $
  *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -201,7 +201,7 @@ librdf_get_query_factory(librdf_world *world,
       if(name && !strcmp(factory->name, name)) {
         break;
       }
-      if(uri && factory->uri && !librdf_uri_equals(factory->uri, uri)) {
+      if(uri && factory->uri && librdf_uri_equals(factory->uri, uri)) {
         break;
       }
     }
@@ -221,8 +221,9 @@ librdf_get_query_factory(librdf_world *world,
 /**
  * librdf_new_query - Constructor - create a new librdf_query object
  * @world: redland world object
- * @name: the query language name
- * @uri: the query language URI (or NULL)
+ * @name: the name identifying the query language
+ * @uri: the URI identifying the query language (or NULL)
+ * @base_uri: the base URI of the query string (or NULL)
  * @query_string: the query string
  *
  * Return value: a new &librdf_query object or NULL on failure
@@ -230,14 +231,16 @@ librdf_get_query_factory(librdf_world *world,
 librdf_query*
 librdf_new_query (librdf_world *world,
                   const char *name, librdf_uri *uri,
-                  const unsigned char *query_string) {
+                  const unsigned char *query_string,
+                  librdf_uri* base_uri) {
   librdf_query_factory* factory;
 
   factory=librdf_get_query_factory(world, name, uri);
   if(!factory)
     return NULL;
 
-  return librdf_new_query_from_factory(world, factory, name, uri, query_string);
+  return librdf_new_query_from_factory(world, factory, name, uri, 
+                                       query_string, base_uri);
 }
 
 
@@ -303,6 +306,7 @@ librdf_new_query_from_query(librdf_query* old_query)
  * @name: query language name
  * @uri: query language URI (or NULL)
  * @query_string: the query string
+ * @base_uri: base URI of the query string (or NULL)
  *
  * Return value: a new &librdf_query object or NULL on failure
  */
@@ -310,7 +314,8 @@ librdf_query*
 librdf_new_query_from_factory(librdf_world *world,
                               librdf_query_factory* factory,
                               const char *name, librdf_uri *uri,
-                              const unsigned char *query_string) {
+                              const unsigned char *query_string,
+                              librdf_uri *base_uri) {
   librdf_query* query;
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(factory, librdf_query_factory, NULL);
@@ -337,7 +342,7 @@ librdf_new_query_from_factory(librdf_world *world,
   
   query->factory=factory;
   
-  if(factory->init(query, name, uri, query_string)) {
+  if(factory->init(query, name, uri, query_string, base_uri)) {
     librdf_free_query(query);
     return NULL;
   }
@@ -463,7 +468,9 @@ main(int argc, char *argv[])
   librdf_uri *uri;
   char *program=argv[0];
   librdf_world *world;
-
+  librdf_uri* format_uri;
+  size_t string_length;
+  unsigned char *string;
   char *query_string=QUERY_STRING;
 
   world=librdf_new_world();
@@ -493,7 +500,7 @@ main(int argc, char *argv[])
 
   fprintf(stdout, "%s: Creating query\n", program);
   query=librdf_new_query(world, QUERY_LANGUAGE,
-                         NULL, (const unsigned char*)query_string);
+                         NULL, (const unsigned char*)query_string, NULL);
   if(!query) {
     fprintf(stderr, "%s: Failed to create new query\n", program);
     return(1);
@@ -539,6 +546,29 @@ main(int argc, char *argv[])
 
   librdf_free_query_results(results);
 
+
+  fprintf(stdout, "%s: Executing a second time\n", program);
+  if(!(results=librdf_model_query_execute(model, query))) {
+    fprintf(stderr, "%s: Second query of model with '%s' failed\n", 
+            program, query_string);
+    return 1;
+  }
+
+  format_uri=librdf_new_uri(world, (unsigned const char*)"http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/");
+  string_length=0;
+
+  string=librdf_query_results_to_counted_string(results, 
+                                                format_uri, NULL,
+                                                &string_length);
+  fprintf(stdout, "%s: Got query results string of length %d\n", program,
+          string_length);
+
+  librdf_free_uri(format_uri);
+  free(string);
+  
+  librdf_free_query_results(results);
+
+
   fprintf(stdout, "%s: Freeing query\n", program);
   librdf_free_query(query);
 
