@@ -2,9 +2,9 @@
  *
  * rdf_query.h - RDF Query Adaptor Factory and Query interfaces and definitions
  *
- * $Id: rdf_query.h,v 1.17 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_query.h,v 1.21 2005/01/31 22:21:56 cmdjb Exp $
  *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -70,7 +70,7 @@ struct librdf_query_factory_s {
   size_t context_length;
   
   /* create a new query */
-  int (*init)(librdf_query* query, const char *name, librdf_uri *uri, const unsigned char *query_string);
+  int (*init)(librdf_query* query, const char *name, librdf_uri *uri, const unsigned char *query_string, librdf_uri *base_uri);
   
   /* copy a query */
   /* clone is assumed to do leave the new query in the same state
@@ -114,6 +114,20 @@ struct librdf_query_factory_s {
 
   /* tidy up query results - OPTIONAL */
   void (*free_results)(librdf_query_results* query_results);
+
+  /* turn the query results into a new string */
+  unsigned char* (*results_to_counted_string)(librdf_query_results *query_results, librdf_uri *format_uri, librdf_uri *base_uri, size_t *length_p);
+
+  /* write the query results to an existing file handle */
+  int (*results_to_file_handle)(librdf_query_results *query_results, FILE *handle, librdf_uri *format_uri, librdf_uri *base_uri);
+
+  /* Return true/false if result is format given - OPTIONAL */
+  int (*results_is_bindings)(librdf_query_results *query_results);
+  int (*results_is_boolean)(librdf_query_results *query_results);
+  int (*results_is_graph)(librdf_query_results *query_results);
+  
+  /* get a boolean result - OPTIONAL */
+  int (*results_get_boolean)(librdf_query_results *query_results);
 };
 
 
@@ -140,9 +154,9 @@ void librdf_query_remove_query_result(librdf_query *query, librdf_query_results*
 REDLAND_API void librdf_query_register_factory(librdf_world *world, const char *name, const unsigned char *uri_string, void (*factory) (librdf_query_factory*));
 
 /* constructor */
-REDLAND_API librdf_query* librdf_new_query(librdf_world* world, const char *name, librdf_uri* uri, const unsigned char *query_string);
+REDLAND_API librdf_query* librdf_new_query(librdf_world* world, const char *name, librdf_uri* uri, const unsigned char *query_string, librdf_uri* base_uri);
 REDLAND_API librdf_query* librdf_new_query_from_query (librdf_query* old_query);
-REDLAND_API librdf_query* librdf_new_query_from_factory(librdf_world* world, librdf_query_factory* factory, const char *name, librdf_uri* uri, const unsigned char* query_string);
+REDLAND_API librdf_query* librdf_new_query_from_factory(librdf_world* world, librdf_query_factory* factory, const char *name, librdf_uri* uri, const unsigned char* query_string, librdf_uri* base_uri);
 
 /* destructor */
 REDLAND_API void librdf_free_query(librdf_query *query);
@@ -162,9 +176,20 @@ REDLAND_API librdf_node* librdf_query_results_get_binding_value(librdf_query_res
 REDLAND_API const char* librdf_query_results_get_binding_name(librdf_query_results* query_results, int offset);
 REDLAND_API librdf_node* librdf_query_results_get_binding_value_by_name(librdf_query_results* query_results, const char *name);
 REDLAND_API int librdf_query_results_get_bindings_count(librdf_query_results* query_results);
+REDLAND_API unsigned char* librdf_query_results_to_counted_string(librdf_query_results *query_results, librdf_uri *format_uri, librdf_uri *base_uri, size_t *length_p);
+REDLAND_API unsigned char* librdf_query_results_to_string(librdf_query_results *query_results, librdf_uri *format_uri, librdf_uri *base_uri);
+REDLAND_API int librdf_query_results_to_file_handle(librdf_query_results *query_results, FILE *handle, librdf_uri *format_uri, librdf_uri *base_uri);
+REDLAND_API int librdf_query_results_to_file(librdf_query_results *query_results, const char *name, librdf_uri *format_uri, librdf_uri *base_uri);
 
 REDLAND_API void librdf_free_query_results(librdf_query_results* query_results);
 
+REDLAND_API int librdf_query_results_is_bindings(librdf_query_results *query_results);
+REDLAND_API int librdf_query_results_is_boolean(librdf_query_results *query_results);
+REDLAND_API int librdf_query_results_is_graph(librdf_query_results *query_results);
+
+REDLAND_API int librdf_query_results_get_boolean(librdf_query_results *query_results);
+
+
 #ifdef __cplusplus
 }
 #endif
