@@ -2,9 +2,9 @@
  *
  * rdf_query_rasqal.c - RDF Query with Rasqal
  *
- * $Id: rdf_query_rasqal.c,v 1.20 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_query_rasqal.c,v 1.29 2005/02/01 00:16:25 cmdjb Exp $
  *
- * Copyright (C) 2004-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -103,7 +103,8 @@ librdf_query_rasqal_warning_handler(void *data, raptor_locator *locator,
 static int
 librdf_query_rasqal_init(librdf_query* query, 
                          const char *name, librdf_uri* uri,
-                         const unsigned char* query_string)
+                         const unsigned char* query_string,
+                         librdf_uri *base_uri)
 {
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
   int len;
@@ -131,8 +132,8 @@ librdf_query_rasqal_init(librdf_query* query,
   strcpy((char*)query_string_copy, (const char*)query_string);
 
   context->query_string=query_string_copy;
-  if(uri)
-    context->uri=librdf_new_uri_from_uri(uri);
+  if(base_uri)
+    context->uri=librdf_new_uri_from_uri(base_uri);
 
   return 0;
 }
@@ -305,16 +306,20 @@ typedef struct {
 } rasqal_redland_triples_match_context;
 
 
-static int
+static rasqal_triple_parts
 rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
                           void *user_data,
-                          rasqal_variable* bindings[4]) 
+                          rasqal_variable* bindings[4],
+                          rasqal_triple_parts parts) 
 {
   rasqal_redland_triples_match_context* rtmc=(rasqal_redland_triples_match_context*)rtm->user_data;
   rasqal_literal* l;
-  librdf_statement* statement=librdf_stream_get_object(rtmc->stream);
+  librdf_statement* statement;
+  rasqal_triple_parts result=(rasqal_triple_parts)0;
+
+  statement=librdf_stream_get_object(rtmc->stream);
   if(!statement)
-    return 1;
+    return (rasqal_triple_parts)0;
   
 #ifdef RASQAL_DEBUG
   LIBRDF_DEBUG1("  matched statement ");
@@ -324,34 +329,36 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
 
   /* set 1 or 2 variable values from the fields of statement */
 
-  if(bindings[0]) {
+  if(bindings[0] && (parts & RASQAL_TRIPLE_SUBJECT)) {
     LIBRDF_DEBUG1("binding subject to variable\n");
     l=redland_node_to_rasqal_literal(librdf_statement_get_subject(statement));
     rasqal_variable_set_value(bindings[0], rasqal_literal_as_node(l));
     rasqal_free_literal(l);
+    result= RASQAL_TRIPLE_SUBJECT;
   }
 
-  if(bindings[1]) {
+  if(bindings[1] && (parts & RASQAL_TRIPLE_PREDICATE)) {
     if(bindings[0] == bindings[1]) {
       if(!librdf_node_equals(librdf_statement_get_subject(statement),
                              librdf_statement_get_predicate(statement)))
-        return 1;
+        return (rasqal_triple_parts)0;
       LIBRDF_DEBUG1("subject and predicate values match\n");
     } else {
       LIBRDF_DEBUG1("binding predicate to variable\n");
       l=redland_node_to_rasqal_literal(librdf_statement_get_predicate(statement));
       rasqal_variable_set_value(bindings[1], rasqal_literal_as_node(l));
       rasqal_free_literal(l);
+      result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_PREDICATE);
     }
   }
 
-  if(bindings[2]) {
+  if(bindings[2] && (parts & RASQAL_TRIPLE_OBJECT)) {
     int bind=1;
     
     if(bindings[0] == bindings[2]) {
       if(!librdf_node_equals(librdf_statement_get_subject(statement),
                              librdf_statement_get_object(statement)))
-        return 1;
+        return (rasqal_triple_parts)0;
       bind=0;
       LIBRDF_DEBUG1("subject and object values match\n");
     }
@@ -360,7 +367,7 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
        ) {
       if(!librdf_node_equals(librdf_statement_get_predicate(statement),
                              librdf_statement_get_object(statement)))
-        return 1;
+        return (rasqal_triple_parts)0;
       bind=0;
       LIBRDF_DEBUG1("predicate and object values match\n");
     }
@@ -370,12 +377,22 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
       l=redland_node_to_rasqal_literal(librdf_statement_get_object(statement));
       rasqal_variable_set_value(bindings[2], rasqal_literal_as_node(l));
       rasqal_free_literal(l);
+      result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_OBJECT);
     }
   }
 
   /* FIXME contexts */
+  /*
+  if(bindings[3] && (parts & RASQAL_TRIPLE_ORIGIN)) {
+    l=redland_node_to_rasqal_literal((librdf_node*)librdf_stream_get_context(rtmc->stream));
+    LIBRDF_DEBUG1("binding origin to variable\n");
+    rasqal_variable_set_value(bindings[3], rasqal_literal_as_node(l));
+    rasqal_free_literal(l);
+    result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_ORIGIN);
+  }
+  */
 
-  return 0;
+  return result;
 }
 
 
@@ -515,13 +532,6 @@ librdf_query_rasqal_execute(librdf_query* query, librdf_model* model)
                           (raptor_uri*)context->uri))
     return NULL;
 
-  if(rasqal_version_decimal < 904) { 
-    /* FIXME: Remove this when Rasqal 0.9.4 is out */
-    /* This ensures query->sources is not NULL but contains no sources */
-    rasqal_query_add_source(context->rq, NULL);
-    raptor_sequence_unshift(rasqal_query_get_source_sequence(context->rq));
-  }
-
   context->results=rasqal_query_execute(context->rq);
   
   results=(librdf_query_results*)LIBRDF_MALLOC(librdf_query_results, sizeof(librdf_query_results));
@@ -545,6 +555,10 @@ librdf_query_rasqal_results_next(librdf_query_results *query_results)
 {
   librdf_query *query=query_results->query;
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return 1;
+  
   return rasqal_query_results_next(context->results);
 }
 
@@ -554,6 +568,10 @@ librdf_query_rasqal_results_finished(librdf_query_results *query_results)
 {
   librdf_query *query=query_results->query;
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return 1;
+  
   return rasqal_query_results_finished(context->results);
 }
 
@@ -569,6 +587,9 @@ librdf_query_rasqal_results_get_bindings(librdf_query_results *query_results,
   int rc;
   int i;
 
+  if(!context->results)
+    return 1;
+  
   if(values) {
     rc=rasqal_query_results_get_bindings(context->results, (const unsigned char ***)names, &literals);
   } else
@@ -602,6 +623,10 @@ librdf_query_rasqal_results_get_binding_name(librdf_query_results *query_results
 {
   librdf_query *query=query_results->query;
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return NULL;
+  
   return (const char*)rasqal_query_results_get_binding_name(context->results, offset);
 }
 
@@ -613,6 +638,9 @@ librdf_query_rasqal_results_get_binding_value_by_name(librdf_query_results *quer
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
   rasqal_literal* literal;
 
+  if(!context->results)
+    return NULL;
+  
   literal=rasqal_query_results_get_binding_value_by_name(context->results, (const unsigned char*)name);
 
   return rasqal_literal_to_redland_node(query->world, literal);
@@ -625,6 +653,9 @@ librdf_query_rasqal_results_get_bindings_count(librdf_query_results *query_resul
   librdf_query *query=query_results->query;
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
 
+  if(!context->results)
+    return 1;
+  
   return rasqal_query_results_get_bindings_count(context->results);
 }
 
@@ -634,10 +665,324 @@ librdf_query_rasqal_free_results(librdf_query_results* query_results)
 {
   librdf_query *query=query_results->query;
   librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return;
+  
   rasqal_free_query_results(context->results);
 }
 
 
+static unsigned char*
+librdf_query_rasqal_results_to_counted_string(librdf_query_results *query_results,
+                                              librdf_uri *format_uri,
+                                              librdf_uri *base_uri,
+                                              size_t *length_p) 
+{
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  unsigned char *string=NULL;
+  size_t string_length=0;
+  raptor_iostream *iostr;
+
+  if(!context->results)
+    return NULL;
+  
+  iostr=raptor_new_iostream_to_string((void**)&string, &string_length, malloc);
+  if(!iostr)
+    return NULL;
+              
+  rasqal_query_results_write(iostr, context->results,
+                             (raptor_uri*)format_uri, (raptor_uri*)base_uri);
+
+  raptor_free_iostream(iostr);
+
+  if(length_p)
+    *length_p=string_length;
+  
+  return string;
+}
+
+
+static int
+librdf_query_rasqal_results_to_file_handle(librdf_query_results *query_results, 
+                                           FILE *handle,
+                                           librdf_uri *format_uri,
+                                           librdf_uri *base_uri) {
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  raptor_iostream *iostr;
+
+  if(!context->results)
+    return 1;
+  
+  iostr=raptor_new_iostream_to_file_handle(handle);
+  if(!iostr)
+    return 1;
+              
+  rasqal_query_results_write(iostr, context->results,
+                             (raptor_uri*)format_uri, (raptor_uri*)base_uri);
+
+  raptor_free_iostream(iostr);
+
+  return 0;
+}
+
+
+static int
+librdf_query_rasqal_results_is_bindings(librdf_query_results* query_results) {
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return 1;
+  
+  return rasqal_query_results_is_bindings(context->results);
+}
+  
+
+static int
+librdf_query_rasqal_results_is_boolean(librdf_query_results* query_results) {
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return 1;
+  
+  return rasqal_query_results_is_boolean(context->results);
+}
+
+
+static int
+librdf_query_rasqal_results_is_graph(librdf_query_results* query_results) {
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return 1;
+  
+  return rasqal_query_results_is_graph(context->results);
+}
+
+
+static int
+librdf_query_rasqal_results_get_boolean(librdf_query_results* query_results) {
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+
+  if(!context->results)
+    return -1;
+  
+  return rasqal_query_results_get_boolean(context->results);
+}
+
+
+typedef struct {
+  librdf_query *query;
+  
+  librdf_query_rasqal_context* qcontext; /* query context */
+
+  librdf_statement* statement; /* current statement */
+  int finished;
+} librdf_query_rasqal_stream_context;
+
+
+static int
+librdf_query_rasqal_query_results_end_of_stream(void* context) {
+  librdf_query_rasqal_stream_context* scontext=(librdf_query_rasqal_stream_context*)context;
+
+  return scontext->finished;
+}
+
+static int
+librdf_query_rasqal_query_results_update_statement(void* context)
+{
+  librdf_query_rasqal_stream_context* scontext=(librdf_query_rasqal_stream_context*)context;
+  librdf_world* world=scontext->query->world;
+  librdf_node* node;
+  
+  raptor_statement *rstatement=rasqal_query_results_get_triple(scontext->qcontext->results);
+  if(!rstatement)
+    return 1;
+  
+  scontext->statement=librdf_new_statement(world);
+  if(!scontext->statement)
+    return 1;
+  
+  if(rstatement->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
+    node=librdf_new_node_from_blank_identifier(world, (const unsigned char*)rstatement->subject);
+  } else if (rstatement->subject_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
+    node=librdf_new_node_from_uri_string(world,
+                                         librdf_uri_as_string((librdf_uri*)rstatement->subject));
+  } else {
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, NULL,
+               "Unknown Raptor subject identifier type %d",
+               rstatement->subject_type);
+    librdf_free_statement(scontext->statement);
+    scontext->statement=NULL;
+    return 1;
+  }
+  
+  librdf_statement_set_subject(scontext->statement, node);
+  
+  if(rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
+    /* FIXME - but only a little
+     * Do I really need to do log10(ordinal) [or /10 and count] + 1 ? 
+     * See also librdf_heuristic_gen_name for some code to repurpose.
+     */
+    static char ordinal_buffer[100]; 
+    int ordinal=*(int*)rstatement->predicate;
+    sprintf(ordinal_buffer, "http://www.w3.org/1999/02/22-rdf-syntax-ns#_%d", ordinal);
+    
+    node=librdf_new_node_from_uri_string(world, (const unsigned char*)ordinal_buffer);
+  } else if (rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_PREDICATE ||
+             rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
+    node=librdf_new_node_from_uri_string(world,
+                                         librdf_uri_as_string((librdf_uri*)rstatement->predicate));
+  } else {
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, NULL,
+               "Unknown Raptor predicate identifier type %d", rstatement->predicate_type);
+    librdf_free_statement(scontext->statement);
+    scontext->statement=NULL;
+    return 1;
+  }
+  
+  librdf_statement_set_predicate(scontext->statement, node);
+  
+  
+  if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL ||
+     rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
+    int is_xml_literal = (rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL);
+    librdf_uri *datatype_uri=(librdf_uri*)rstatement->object_literal_datatype;
+    
+    if(is_xml_literal)
+      librdf_statement_set_object(scontext->statement,
+                                  librdf_new_node_from_literal(world,
+                                                               (const unsigned char*)rstatement->object,
+                                                               (const char*)rstatement->object_literal_language,
+                                                               is_xml_literal));
+    else
+      librdf_statement_set_object(scontext->statement,
+                                  librdf_new_node_from_typed_literal(world,
+                                                                     (const unsigned char*)rstatement->object,
+                                                                     (const char*)rstatement->object_literal_language,
+                                                                     datatype_uri));
+    
+  } else if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
+    node=librdf_new_node_from_blank_identifier(world, (const unsigned char*)rstatement->object);
+    librdf_statement_set_object(scontext->statement, node);
+  } else if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
+    node=librdf_new_node_from_uri_string(world,
+                                         librdf_uri_as_string((librdf_uri*)rstatement->object));
+    librdf_statement_set_object(scontext->statement, node);
+  } else {
+    librdf_log(world,
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+               "Unknown Raptor object identifier type %d", rstatement->object_type);
+    librdf_free_statement(scontext->statement);
+    scontext->statement=NULL;
+    return 1;
+  }
+
+  return 0;
+}
+
+
+static int
+librdf_query_rasqal_query_results_next_statement(void* context)
+{
+  librdf_query_rasqal_stream_context* scontext=(librdf_query_rasqal_stream_context*)context;
+
+  if(scontext->finished)
+    return 1;
+  
+  if(scontext->statement) {
+    librdf_free_statement(scontext->statement);
+    scontext->statement=NULL;
+  }
+
+  scontext->finished=rasqal_query_results_next_triple(scontext->qcontext->results);
+  if(!scontext->finished)
+    librdf_query_rasqal_query_results_update_statement(scontext);
+  
+  return scontext->finished;
+}
+
+
+static void*
+librdf_query_rasqal_query_results_get_statement(void* context, int flags) {
+  librdf_query_rasqal_stream_context* scontext=(librdf_query_rasqal_stream_context*)context;
+
+  switch(flags) {
+    case LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT:
+      return scontext->statement;
+
+    case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
+      return NULL;
+      
+    default:
+      librdf_log(scontext->query->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, NULL,
+                 "Unknown iterator method flag %d\n", flags);
+      return NULL;
+  }
+
+}
+
+
+static void
+librdf_query_rasqal_query_results_finished(void* context) {
+  librdf_query_rasqal_stream_context* scontext=(librdf_query_rasqal_stream_context*)context;
+
+  if(scontext) {
+    if(scontext->statement)
+      librdf_free_statement(scontext->statement);
+
+    LIBRDF_FREE(librdf_query_rasqal_context, scontext);
+  }
+}
+
+
+static librdf_stream*
+librdf_query_rasqal_results_as_stream(librdf_query_results* query_results) {
+  librdf_query *query=query_results->query;
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  librdf_query_rasqal_stream_context* scontext;
+  librdf_stream *stream;
+
+  if(!context->results)
+    return NULL;
+  
+  scontext=(librdf_query_rasqal_stream_context*)LIBRDF_CALLOC(librdf_query_rasqal_stream_context, 1, sizeof(librdf_query_rasqal_stream_context));
+  if(!scontext)
+    return NULL;
+
+  scontext->query=query;
+  scontext->qcontext=context;
+
+  if(librdf_query_rasqal_query_results_update_statement(scontext)) {
+    librdf_query_rasqal_query_results_finished((void*)scontext);
+    return NULL;
+  }
+  
+  stream=librdf_new_stream(query->world,
+                           (void*)scontext,
+                           &librdf_query_rasqal_query_results_end_of_stream,
+                           &librdf_query_rasqal_query_results_next_statement,
+                           &librdf_query_rasqal_query_results_get_statement,
+                           &librdf_query_rasqal_query_results_finished);
+  if(!stream) {
+    librdf_query_rasqal_query_results_finished((void*)scontext);
+    return NULL;
+  }
+
+  return stream;  
+}
+
+
+
 /* local function to register list query functions */
 
 static void
@@ -657,6 +1002,14 @@ librdf_query_rasqal_register_factory(librdf_query_factory *factory)
   factory->results_get_binding_value_by_name = librdf_query_rasqal_results_get_binding_value_by_name;
   factory->results_get_bindings_count         = librdf_query_rasqal_results_get_bindings_count;
   factory->free_results                       = librdf_query_rasqal_free_results;
+  factory->results_to_counted_string          = librdf_query_rasqal_results_to_counted_string;
+  factory->results_to_file_handle             = librdf_query_rasqal_results_to_file_handle;
+  factory->results_is_bindings                = librdf_query_rasqal_results_is_bindings;
+  factory->results_is_boolean                 = librdf_query_rasqal_results_is_boolean;
+  factory->results_is_graph                   = librdf_query_rasqal_results_is_graph;
+  factory->results_get_boolean                = librdf_query_rasqal_results_get_boolean;
+  factory->results_as_stream                  = librdf_query_rasqal_results_as_stream;
+
 }
 
 
