@@ -2,7 +2,7 @@
  *
  * rdf_query_triples.c - RDF Query simple triple query language
  *
- * $Id: rdf_query_triples.c,v 1.15 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_query_triples.c,v 1.17 2004/11/09 16:24:43 cmdjb Exp $
  *
  * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -49,7 +49,7 @@ typedef struct
 
 
 /* prototypes for local functions */
-static int librdf_query_triples_init(librdf_query* query, const char *name, librdf_uri* uri, const unsigned char *query_string);
+static int librdf_query_triples_init(librdf_query* query, const char *name, librdf_uri* uri, const unsigned char *query_string, librdf_uri* base_uri);
 static librdf_query_results* librdf_query_triples_query_execute(librdf_query* query, librdf_model* model);
 static librdf_stream* librdf_query_triples_results_as_stream(librdf_query_results* query_results);
 
@@ -129,7 +129,8 @@ librdf_query_triples_find_next_term(unsigned char *string)
 static int
 librdf_query_triples_init(librdf_query* query, 
                           const char *name, librdf_uri* uri,
-                          const unsigned char* query_string)
+                          const unsigned char* query_string, 
+                          librdf_uri* base_uri)
 {
   librdf_query_triples_context *context=(librdf_query_triples_context*)query->context;
   int len;
@@ -165,7 +166,8 @@ librdf_query_triples_init(librdf_query* query,
       return 0;
     }
     librdf_statement_set_subject(&context->statement, subject);
-  }
+  } else
+   subject=NULL;
   cur=p;
   
   /* predicate - NULL or URI */
@@ -190,7 +192,8 @@ librdf_query_triples_init(librdf_query* query,
       return 0;
     }
     librdf_statement_set_predicate(&context->statement, predicate);
-  }
+  } else
+   predicate=NULL;
   cur=p;
   
   /* object - NULL, literal or URI */
@@ -199,8 +202,10 @@ librdf_query_triples_init(librdf_query* query,
     librdf_log(query->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, NULL,
                "Bad triples query language syntax - bad object in '%s'", cur);
     LIBRDF_FREE(cstring, query_string_copy);
-    librdf_free_node(subject);
-    librdf_free_node(predicate);
+    if(subject)
+      librdf_free_node(subject);
+    if(predicate)
+      librdf_free_node(predicate);
     return 0;
   }
   *p='\0';
@@ -215,8 +220,10 @@ librdf_query_triples_init(librdf_query* query,
       object=librdf_new_node_from_uri_string(query->world, cur);
     if(!object) {
       LIBRDF_FREE(cstring, query_string_copy);
-      librdf_free_node(subject);
-      librdf_free_node(predicate);
+      if(subject)
+        librdf_free_node(subject);
+      if(predicate)
+        librdf_free_node(predicate);
       return 0;
     }
     librdf_statement_set_object(&context->statement, object);
