@@ -2,9 +2,9 @@
  *
  * rdf_serializer.c - RDF Serializer Factory implementation
  *
- * $Id: rdf_serializer.c,v 1.36 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_serializer.c,v 1.42 2005/01/30 23:15:25 cmdjb Exp $
  *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -192,14 +192,21 @@ librdf_get_serializer_factory(librdf_world *world,
 	continue;
 
       /* MIME type may need to match */
-      if(mime_type && factory->mime_type &&
-         strcmp(factory->mime_type, mime_type))
-        continue;
+      if(mime_type) {
+        if(!factory->mime_type)
+          continue;
+        if(strcmp(factory->mime_type, mime_type))
+          continue;
+      }
       
       /* URI may need to match */
-      if(type_uri && factory->type_uri &&
-         ! librdf_uri_equals(factory->type_uri, type_uri))
-        continue;
+      if(type_uri) {
+        if(!factory->type_uri)
+          continue;
+        
+        if(librdf_uri_equals(factory->type_uri, type_uri))
+          continue;
+      }
 
       /* found it */
       break;
@@ -333,6 +340,7 @@ librdf_serializer_serialize_model_to_file_handle(librdf_serializer* serializer,
                                                  librdf_model* model) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(handle, FILE*, 1);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
 
   return serializer->factory->serialize_model_to_file_handle(serializer->context,
@@ -524,6 +532,8 @@ librdf_serializer_set_feature(librdf_serializer* serializer,
  * @uri: URI of namespace
  * @prefix: prefix to use
  * 
+ * NOTE: May not be implemented yet - depends on raptor_serializer work.
+ * 
  * Return value: non 0 on failure
  **/
   
@@ -652,10 +662,10 @@ main(int argc, char *argv[])
   string=librdf_serializer_serialize_model_to_counted_string(serializer,
                                                              base_uri, model,
                                                              &string_length);
-#define EXPECTED_BAD_STRING_LENGTH 237
+#define EXPECTED_BAD_STRING_LENGTH 346
   if(string_length != EXPECTED_BAD_STRING_LENGTH) {
     fprintf(stderr, "%s: Serialising to RDF/XML returned string '%s' size %d, expected %d\n", program, string,
-            string_length, EXPECTED_BAD_STRING_LENGTH);
+            (int)string_length, EXPECTED_BAD_STRING_LENGTH);
     return 1;
   }
 
