@@ -2,7 +2,7 @@
  *
  * rdf_storage.c - RDF Storage Implementation
  *
- * $Id: rdf_storage.c,v 1.78 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_storage.c,v 1.81 2004/11/22 23:59:27 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -44,8 +44,6 @@
 
 #include <redland.h>
 #include <rdf_storage.h>
-#include <rdf_storage_hashes.h>
-#include <rdf_storage_list.h>
 
 
 #ifndef STANDALONE
@@ -77,15 +75,21 @@ static librdf_iterator* librdf_storage_node_stream_to_node_create(librdf_storage
 void
 librdf_init_storage(librdf_world *world)
 {
-#ifdef HAVE_MYSQL
+#ifdef STORAGE_MYSQL
   librdf_init_storage_mysql(world);
 #endif
-#ifdef HAVE_TSTORE
+#ifdef STORAGE_TSTORE
   librdf_init_storage_tstore(world);
 #endif
-  /* Always have storage list, hashes, file implementations available */
+#ifdef STORAGE_SQLITE
+  librdf_init_storage_sqlite(world);
+#endif
+#ifdef STORAGE_FILE
   librdf_init_storage_file(world);
+#endif
+  /* Always have storage list, hashes, file implementations available */
   librdf_init_storage_hashes(world);
+  /* Always have storage memory must always be available */
   librdf_init_storage_list(world);
 }
 
@@ -1284,13 +1288,14 @@ librdf_storage_query_execute(librdf_storage* storage, librdf_query *query)
 }
 
 
-void
+int
 librdf_storage_sync(librdf_storage* storage) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(storage, librdf_storage);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
 
   if(storage->factory->sync)
-    storage->factory->sync(storage);
+    return storage->factory->sync(storage);
+  return 0;
 }
 
 
