@@ -2,7 +2,7 @@
  *
  * rdf_storage.h - RDF Storage Factory and Storage interfaces and definitions
  *
- * $Id: rdf_storage.h,v 1.38 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_storage.h,v 1.41 2004/11/22 23:58:22 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -138,7 +138,7 @@ struct librdf_storage_factory_s {
   librdf_stream* (*context_serialise)(librdf_storage* storage, librdf_node* context);
 
   /* synchronise to underlying storage - OPTIONAL */
-  void (*sync)(librdf_storage* storage);
+  int (*sync)(librdf_storage* storage);
 
   /* add statements to the context - OPTIONAL (rdf_storage will do it
    * using context_add_statement if missing)
@@ -169,14 +169,19 @@ struct librdf_storage_factory_s {
 
 #include <rdf_storage_list.h>
 #include <rdf_storage_hashes.h>
+
 void librdf_init_storage_file(librdf_world *world);
 
-#ifdef HAVE_MYSQL
-#include <rdf_storage_mysql.h>
+#ifdef STORAGE_MYSQL
+void librdf_init_storage_mysql(librdf_world *world);
 #endif
 
-#ifdef HAVE_TSTORE
-#include <rdf_storage_tstore.h>
+#ifdef STORAGE_TSTORE
+void librdf_init_storage_tstore(librdf_world *world);
+#endif
+
+#ifdef STORAGE_SQLITE
+void librdf_init_storage_sqlite(librdf_world *world);
 #endif
 
 
@@ -192,6 +197,10 @@ librdf_storage_factory* librdf_get_storage_factory(const char *name);
 void librdf_storage_add_reference(librdf_storage *storage);
 void librdf_storage_remove_reference(librdf_storage *storage);
 
+#ifdef HAVE_SQLITE
+void librdf_init_storage_sqlite(librdf_world *world);
+#endif
+
 #endif
 
 
@@ -252,7 +261,7 @@ REDLAND_API int librdf_storage_supports_query(librdf_storage* storage, librdf_qu
 REDLAND_API librdf_query_results* librdf_storage_query_execute(librdf_storage* storage, librdf_query *query);
 
 /* synchronise a storage to the backing store */
-REDLAND_API void librdf_storage_sync(librdf_storage *storage);
+REDLAND_API int librdf_storage_sync(librdf_storage *storage);
 
 /* find statements in a given context */
 REDLAND_API librdf_stream* librdf_storage_find_statements_in_context(librdf_storage* storage, librdf_statement* statement, librdf_node* context_node);
