@@ -2,26 +2,37 @@
  *
  * rdf_storage_list.c - RDF Storage in memory as a list implementation
  *
- * $Id: rdf_storage_list.c,v 1.42 2004/01/25 18:00:42 cmdjb Exp $
+ * $Id: rdf_storage_list.c,v 1.49 2004/11/01 20:43:35 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bristol.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
  * 
  * 
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include <rdf_config.h>
+#endif
+
+#ifdef WIN32
+#include <win32_rdf_config.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -30,7 +41,7 @@
 #endif
 #include <sys/types.h>
 
-#include <librdf.h>
+#include <redland.h>
 
 
 typedef struct
@@ -228,6 +239,10 @@ librdf_storage_list_size(librdf_storage* storage)
 static int
 librdf_storage_list_add_statement(librdf_storage* storage, librdf_statement* statement)
 {
+  /* Do not add duplicate statements */
+  if(librdf_storage_list_contains_statement(storage, statement))
+    return 0;
+
   return librdf_storage_list_context_add_statement(storage, NULL, statement);
 }
 
@@ -239,29 +254,35 @@ librdf_storage_list_add_statements(librdf_storage* storage,
   librdf_storage_list_context* context=(librdf_storage_list_context*)storage->context;
   int status=0;
 
-  while(!librdf_stream_end(statement_stream)) {
+  for(; !librdf_stream_end(statement_stream);
+      librdf_stream_next(statement_stream)) {
     librdf_statement* statement=librdf_stream_get_object(statement_stream);
     librdf_storage_list_node* sln;
 
-    if(statement) {
-      sln=(librdf_storage_list_node*)LIBRDF_MALLOC(librdf_storage_list_node, sizeof(librdf_storage_list_node));
-      if(!sln) {
-        status=1;
-        break;
-      }
-
-      /* copy shared statement */
-      sln->statement=librdf_new_statement_from_statement(statement);
-      if(!sln->statement) {
-        LIBRDF_FREE(librdf_storage_list_node, sln);
-        status=1;
-        break;
-      }
-      sln->context=NULL;
-      librdf_list_add(context->list, sln);
-    } else
+    if(!statement) {
       status=1;
-    librdf_stream_next(statement_stream);
+      break;
+    }
+
+    /* Do not add duplicate statements */
+    if(librdf_storage_list_contains_statement(storage, statement))
+      continue;
+
+    sln=(librdf_storage_list_node*)LIBRDF_MALLOC(librdf_storage_list_node, sizeof(librdf_storage_list_node));
+    if(!sln) {
+      status=1;
+      break;
+    }
+    
+    /* copy shared statement */
+    sln->statement=librdf_new_statement_from_statement(statement);
+    if(!sln->statement) {
+      LIBRDF_FREE(librdf_storage_list_node, sln);
+      status=1;
+      break;
+    }
+    sln->context=NULL;
+    librdf_list_add(context->list, sln);
   }
   
   return status;
@@ -381,7 +402,9 @@ librdf_storage_list_serialise_get_statement(void* context, int flags)
       else
         return NULL;
     default:
-      LIBRDF_ERROR2(scontext->iterator->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(scontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       return NULL;
   }
 }
@@ -450,6 +473,12 @@ librdf_storage_list_context_add_statement(librdf_storage* storage,
   librdf_storage_list_node* sln;
   int status;
 
+  if(context_node && !context->index_contexts) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
+    return 1;
+  }
+  
   /* Store statement + node in the storage_list */
   sln=(librdf_storage_list_node*)LIBRDF_MALLOC(librdf_storage_list_node, sizeof(librdf_storage_list_node));
   if(!sln)
@@ -519,6 +548,12 @@ librdf_storage_list_context_remove_statement(librdf_storage* storage,
   size_t size;
   int status;
 
+  if(context_node && !context->index_contexts) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
+    return 1;
+  }
+  
   search_sln.statement=statement;
   search_sln.context=context_node;
 
@@ -579,8 +614,11 @@ librdf_storage_list_context_serialise(librdf_storage* storage,
   librdf_stream* stream;
   size_t size;
 
-  if(!context->index_contexts)
+  if(!context->index_contexts) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
     return NULL;
+  }
   
   scontext=(librdf_storage_list_context_serialise_stream_context*)LIBRDF_CALLOC(librdf_storage_list_context_serialise_stream_context, 1, sizeof(librdf_storage_list_context_serialise_stream_context));
   if(!scontext)
@@ -674,7 +712,9 @@ librdf_storage_list_context_serialise_get_statement(void* context, int flags)
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
       return scontext->context_node;
     default:
-      LIBRDF_ERROR2(scontext->iterator->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(scontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       return NULL;
   }
   
@@ -769,7 +809,9 @@ librdf_storage_list_get_contexts_get_method(void* iterator, int flags)
       break;
       
     default:
-      LIBRDF_ERROR2(icontext->storage->world, "Unknown iterator method flag %d\n", flags);
+      librdf_log(icontext->iterator->world,
+                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+                 "Unknown iterator method flag %d\n", flags);
       result=NULL;
       break;
   }
@@ -811,8 +853,11 @@ librdf_storage_list_get_contexts(librdf_storage* storage)
   librdf_storage_list_get_contexts_iterator_context* icontext;
   librdf_iterator* iterator;
 
-  if(!context->index_contexts)
+  if(!context->index_contexts) {
+    librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
+               "Storage was created without context support");
     return NULL;
+  }
   
   icontext=(librdf_storage_list_get_contexts_iterator_context*)LIBRDF_CALLOC(librdf_storage_list_get_contexts_iterator_context, 1, sizeof(librdf_storage_list_get_contexts_iterator_context));
   if(!icontext)
@@ -908,8 +953,8 @@ librdf_storage_list_register_factory(librdf_storage_factory *factory)
 
 
 void
-librdf_init_storage_list(void)
+librdf_init_storage_list(librdf_world *world)
 {
-  librdf_storage_register_factory("memory", "In memory",
+  librdf_storage_register_factory(world, "memory", "In memory",
                                   &librdf_storage_list_register_factory);
 }
