@@ -2,7 +2,7 @@
  *
  * redland.h - Redland RDF Application Framework main header
  *
- * $Id: redland.h,v 1.10 2004/11/01 20:43:35 cmdjb Exp $
+ * $Id: redland.h,v 1.11 2004/11/10 10:27:25 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -33,6 +33,7 @@
 extern "C" {
 #endif
 
+#ifndef LIBRDF_OBJC_FRAMEWORK
 /* Redland consists of */
 /* raptor */
 #include <raptor.h>
@@ -40,6 +41,11 @@ extern "C" {
 #include <rasqal.h>
 /* librdf */
 #include <librdf.h>
+#else
+#include <Redland/raptor.h>
+#include <Redland/rasqal.h>
+#include <Redland/librdf.h>
+#endif
 
 #ifdef __cplusplus
 }
