@@ -2,9 +2,9 @@
  *
  * raptor_namespace.c - Raptor XML namespace classes
  *
- * $Id: raptor_namespace.c,v 1.41 2004/10/21 16:29:36 cmdjb Exp $
+ * $Id: raptor_namespace.c,v 1.51 2005/01/15 17:18:53 cmdjb Exp $
  *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -92,12 +92,12 @@
  *
  */
 
-const char * const raptor_xml_namespace_uri=(const char *)"http://www.w3.org/XML/1998/namespace";
-const char * const raptor_rdf_namespace_uri=(const char *)RAPTOR_RDF_MS_URI;
+const unsigned char * const raptor_xml_namespace_uri=(const unsigned char *)"http://www.w3.org/XML/1998/namespace";
+const unsigned char * const raptor_rdf_namespace_uri=(const unsigned char *)"http://www.w3.org/1999/02/22-rdf-syntax-ns#";
 const unsigned int raptor_rdf_namespace_uri_len=43;
-const char * const raptor_rdf_schema_namespace_uri=(const char *)RAPTOR_RDF_SCHEMA_URI;
-const char * const raptor_xmlschema_datatypes_namespace_uri=(const char *)RAPTOR_XMLSCHEMA_DATATYPES_URI;
-const char * const raptor_owl_namespace_uri=(const char *)RAPTOR_OWL_URI;
+const unsigned char * const raptor_rdf_schema_namespace_uri=(const unsigned char *)"http://www.w3.org/2000/01/rdf-schema#";
+const unsigned char * const raptor_xmlschema_datatypes_namespace_uri=(const unsigned char *)"http://www.w3.org/2001/XMLSchema#";
+const unsigned char * const raptor_owl_namespace_uri=(const unsigned char *)"http://www.w3.org/2002/07/owl#";
 
 
 /**
@@ -191,7 +191,6 @@ raptor_namespaces_start_namespace_full(raptor_namespace_stack *nstack,
                                        const unsigned char *prefix, 
                                        const unsigned char *ns_uri_string,
                                        int depth)
-
 {
   raptor_namespace *ns;
 
@@ -205,7 +204,8 @@ raptor_namespaces_start_namespace_full(raptor_namespace_stack *nstack,
 
 
 void
-raptor_namespaces_clear(raptor_namespace_stack *nstack) {
+raptor_namespaces_clear(raptor_namespace_stack *nstack)
+{
   raptor_namespace *ns=nstack->top;
   while(ns) {
     raptor_namespace* next_ns=ns->next;
@@ -226,7 +226,8 @@ raptor_namespaces_clear(raptor_namespace_stack *nstack) {
 
 
 void
-raptor_free_namespaces(raptor_namespace_stack *nstack) {
+raptor_free_namespaces(raptor_namespace_stack *nstack)
+{
   raptor_namespaces_clear(nstack);
   RAPTOR_FREE(raptor_namespace_stack, nstack);
 }
@@ -278,6 +279,20 @@ raptor_namespaces_find_namespace(raptor_namespace_stack *nstack,
 }
 
 
+raptor_namespace*
+raptor_namespaces_find_namespace_by_uri(raptor_namespace_stack *nstack, 
+                                        raptor_uri *ns_uri)
+{
+  raptor_namespace* ns;
+  
+  for(ns=nstack->top; ns ; ns=ns->next)
+    if(nstack->uri_handler->uri_equals(nstack->uri_context, ns->uri, ns_uri))
+      return ns;
+
+  return NULL;
+}
+
+
 int
 raptor_namespaces_namespace_in_scope(raptor_namespace_stack *nstack, 
                                      const raptor_namespace *nspace)
@@ -292,26 +307,22 @@ raptor_namespaces_namespace_in_scope(raptor_namespace_stack *nstack,
 
 
 raptor_namespace*
-raptor_new_namespace(raptor_namespace_stack *nstack,
-                     const unsigned char *prefix, 
-                     const unsigned char *ns_uri_string, int depth)
+raptor_new_namespace_from_uri(raptor_namespace_stack *nstack,
+                              const unsigned char *prefix, 
+                              raptor_uri* ns_uri, int depth)
 {
   int prefix_length=0;
   int len;
   raptor_namespace *ns;
   unsigned char *p;
 
-  /* Convert an empty namespace string "" to a NULL pointer */
-  if(ns_uri_string && !*ns_uri_string)
-    ns_uri_string=NULL;
-
 #ifndef STANDALONE
 #if RAPTOR_DEBUG >1
-  RAPTOR_DEBUG4("namespace prefix %s uri %s depth %d\n", prefix ? (char*)prefix : "(default)", ns_uri_string ? (char*)ns_uri_string : "(none)", depth);
+  RAPTOR_DEBUG4("namespace prefix %s uri %s depth %d\n", prefix ? (char*)prefix : "(default)", ns_uri ? (char*)raptor_uri_as_string(ns_uri) : "(none)", depth);
 #endif
 #endif
 
-  if(prefix && !ns_uri_string) {
+  if(prefix && !ns_uri) {
     /* failed to find namespace - now what? */
     if(nstack->error_handler)
       nstack->error_handler((raptor_parser*)nstack->error_data, "The namespace URI for prefix \"%s\" is empty.", prefix);
@@ -331,8 +342,8 @@ raptor_new_namespace(raptor_namespace_stack *nstack,
     return NULL;
 
   p=(unsigned char*)ns+sizeof(raptor_namespace);
-  if(ns_uri_string) {
-    ns->uri=(*nstack->uri_handler->new_uri)(nstack->uri_context, ns_uri_string);
+  if(ns_uri) {
+    ns->uri=(*nstack->uri_handler->uri_copy)(nstack->uri_context, ns_uri);
     if(!ns->uri) {
       RAPTOR_FREE(raptor_namespace, ns);
       return NULL;
@@ -348,7 +359,7 @@ raptor_new_namespace(raptor_namespace_stack *nstack,
   ns->depth=depth;
 
   /* set convienience flags when there is a defined namespace URI */
-  if(ns_uri_string) {
+  if(ns->uri) {
     if(nstack->uri_handler->uri_equals(nstack->uri_context, ns->uri, nstack->rdf_ms_uri))
       ns->is_rdf_ms=1;
     else if(nstack->uri_handler->uri_equals(nstack->uri_context, ns->uri, nstack->rdf_schema_uri))
@@ -360,6 +371,30 @@ raptor_new_namespace(raptor_namespace_stack *nstack,
   return ns;
 }
 
+
+raptor_namespace*
+raptor_new_namespace(raptor_namespace_stack *nstack,
+                     const unsigned char *prefix, 
+                     const unsigned char *ns_uri_string, int depth)
+{
+  raptor_uri* ns_uri=NULL;
+  raptor_namespace* ns;
+
+  /* Convert an empty namespace string "" to a NULL pointer */
+  if(ns_uri_string && !*ns_uri_string)
+    ns_uri_string=NULL;
+
+  if(ns_uri_string)
+    ns_uri=raptor_new_uri(ns_uri_string);
+  ns=raptor_new_namespace_from_uri(nstack, prefix, ns_uri, depth);
+  if(ns_uri)
+    raptor_free_uri(ns_uri);
+
+  return ns;
+}
+
+
+
 /* copy to a new stack with a new depth */
 int
 raptor_namespace_copy(raptor_namespace_stack *nstack,
@@ -369,9 +404,7 @@ raptor_namespace_copy(raptor_namespace_stack *nstack,
 {
   raptor_namespace *new_ns;
 
-  unsigned char *uri_string=(*nstack->uri_handler->uri_as_string)(nstack->uri_context, ns->uri);
-
-  new_ns=raptor_new_namespace(nstack, ns->prefix, uri_string, new_depth);
+  new_ns=raptor_new_namespace_from_uri(nstack, ns->prefix, ns->uri, new_depth);
   if(!new_ns)
     return 1;
   
@@ -407,11 +440,15 @@ raptor_namespace_get_prefix(const raptor_namespace *ns)
 unsigned char *
 raptor_namespaces_format(const raptor_namespace *ns, size_t *length_p)
 {
-  size_t uri_length;
-  const unsigned char *uri_string=raptor_uri_as_counted_string(ns->uri, &uri_length);
-  size_t length=8+uri_length+ns->prefix_length; /* 8=length of [[xmlns=""] */
+  size_t uri_length=0L;
+  const unsigned char *uri_string=NULL;
+  size_t length;
   unsigned char *buffer;
 
+  if(ns->uri)
+    uri_string=raptor_uri_as_counted_string(ns->uri, &uri_length);
+  length=8+uri_length+ns->prefix_length; /* 8=length of [[xmlns=""] */
+
   if(ns->prefix)
     length++; /* for : */
   
@@ -438,6 +475,109 @@ raptor_namespaces_format(const raptor_namespace *ns, size_t *length_p)
 }
 
 
+int
+raptor_iostream_write_namespace(raptor_iostream* iostr, raptor_namespace *ns)
+{
+  size_t uri_length=0L;
+  const unsigned char *uri_string=NULL;
+  
+  if(!ns || !iostr)
+    return 1;
+  
+  if(ns->uri)
+    uri_string=raptor_uri_as_counted_string(ns->uri, &uri_length);
+  
+  raptor_iostream_write_counted_string(iostr, "xmlns", 5);
+  if(ns->prefix) {
+    raptor_iostream_write_byte(iostr, ':');
+    raptor_iostream_write_string(iostr, ns->prefix);
+  }
+  raptor_iostream_write_counted_string(iostr, "=\"", 2);
+  if(uri_length)
+    raptor_iostream_write_counted_string(iostr, uri_string, uri_length);
+  raptor_iostream_write_byte(iostr, '"');
+
+  return 0;
+}
+
+
+int
+raptor_new_namespace_parts_from_string(const unsigned char *string,
+                                       unsigned char **prefix,
+                                       unsigned char **uri_string)
+{
+  const unsigned char *t;
+  unsigned char quote;
+  
+  if((!prefix || !uri_string))
+    return 1;
+  
+  if(!string || (string && !*string))
+    return 1;
+
+  if(strncmp((const char*)string, "xmlns", 5))
+    return 1;
+
+  *prefix=NULL;
+  *uri_string=NULL;
+
+  /*
+   * Four cases are expected and handled:
+   * xmlns=""
+   * xmlns="uri"
+   * xmlns:foo=""
+   * xmlns:foo="uri"
+   *
+   * (with " or ' quotes)
+   */
+
+  /* skip "xmlns" */
+  string+= 5;
+  
+  if (*string == ':') {
+    /* non-empty prefix */
+    t= ++string;
+    while(*string && *string != '=')
+      string++;
+    if(!*string || string == t)
+      return 1;
+
+    *prefix=(unsigned char*)RAPTOR_MALLOC(cstring, string-t+1);
+    if(!*prefix)
+      return 1;
+    strncpy((char*)*prefix, (const char*)t, string-t);
+    (*prefix)[string-t]='\0';
+  }
+
+  if(*string++ != '=')
+    return 1;
+
+  if(*string != '"' && *string != '\'')
+    return 1;
+  quote=*string++;
+
+  t=string;
+  while(*string && *string != quote)
+    string++;
+
+  if(*string != quote)
+    return 1;
+
+  if(!(string-t))
+    /* xmlns...="" */
+    *uri_string=NULL;
+  else {
+    *uri_string=(unsigned char*)RAPTOR_MALLOC(cstring, string-t+1);
+    if(!*uri_string)
+      return 1;
+    strncpy((char*)*uri_string, (const char*)t, string-t);
+    (*uri_string)[string-t]='\0';
+  }
+  
+  return 0;
+}
+
+
 #ifdef RAPTOR_DEBUG
 void
 raptor_namespace_print(FILE *stream, raptor_namespace* ns) 
