@@ -2,9 +2,9 @@
  *
  * rasqal_expr.c - Rasqal general expression support
  *
- * $Id: rasqal_expr.c,v 1.77 2004/10/29 11:24:27 cmdjb Exp $
+ * $Id: rasqal_expr.c,v 1.90 2005/02/03 22:26:58 cmdjb Exp $
  *
- * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -87,7 +87,7 @@ rasqal_new_variable(rasqal_query* rq,
     }
   }
     
-  v=(rasqal_variable*)calloc(sizeof(rasqal_variable), 1);
+  v=(rasqal_variable*)RASQAL_CALLOC(rasqal_variable, sizeof(rasqal_variable), 1);
 
   v->name=name;
   v->value=value;
@@ -151,6 +151,8 @@ rasqal_variable_set_value(rasqal_variable* v, rasqal_literal *e)
     rasqal_free_literal(v->value);
   v->value=e;
 #ifdef RASQAL_DEBUG
+  if(!v->name)
+    RASQAL_FATAL1("variable has no name");
   RASQAL_DEBUG2("setting variable %s to value ", v->name);
   if(v->value)
     rasqal_literal_print(v->value, stderr);
@@ -195,7 +197,7 @@ rasqal_prefix_print(rasqal_prefix* p, FILE* fh)
 rasqal_triple*
 rasqal_new_triple(rasqal_literal* subject, rasqal_literal* predicate, rasqal_literal* object)
 {
-  rasqal_triple* t=(rasqal_triple*)calloc(sizeof(rasqal_triple), 1);
+  rasqal_triple* t=(rasqal_triple*)RASQAL_CALLOC(rasqal_triple,sizeof(rasqal_triple), 1);
 
   t->subject=subject;
   t->predicate=predicate;
@@ -204,6 +206,17 @@ rasqal_new_triple(rasqal_literal* subject, rasqal_literal* predicate, rasqal_lit
   return t;
 }
 
+rasqal_triple*
+rasqal_new_triple_from_triple(rasqal_triple* t) {
+  rasqal_triple* newt=(rasqal_triple*)RASQAL_CALLOC(rasqal_triple,sizeof(rasqal_triple), 1);
+
+  newt->subject=rasqal_new_literal_from_literal(t->subject);
+  newt->predicate=rasqal_new_literal_from_literal(t->predicate);
+  newt->object=rasqal_new_literal_from_literal(t->object);
+
+  return newt;
+}
+
 void
 rasqal_free_triple(rasqal_triple* t)
 {
@@ -217,6 +230,13 @@ rasqal_free_triple(rasqal_triple* t)
 }
 
 
+static const char *rasqal_triple_flag_strings[RASQAL_TRIPLE_FLAGS_LAST+1]={
+  NULL,
+  "EXACT",
+  "OPTIONAL"
+};
+
+
 void
 rasqal_triple_print(rasqal_triple* t, FILE* fh)
 {
@@ -232,6 +252,24 @@ rasqal_triple_print(rasqal_triple* t, FILE* fh)
     rasqal_literal_print(t->origin, fh);
     fputc(')', fh);
   }
+  if(t->flags) {
+    unsigned int f=t->flags;
+    unsigned int b=1;
+    int seen=0;
+    
+    fputs(" with flags(", fh);
+    while(f && b <= RASQAL_TRIPLE_FLAGS_LAST) {
+      if(seen)
+        fputs(", ", fh);
+      if(f & b) {
+        fputs(rasqal_triple_flag_strings[b], fh);
+        seen=1;
+        f &= ~b;
+      }
+      b <<= 1;
+    }
+    fputc(')', fh);
+  }
 }
 
 
@@ -249,10 +287,24 @@ rasqal_triple_get_origin(rasqal_triple* t)
 }
 
 
+void
+rasqal_triple_set_flags(rasqal_triple* t, unsigned int flags)
+{
+  t->flags=flags;
+}
+
+
+unsigned int
+rasqal_triple_get_flags(rasqal_triple* t)
+{
+  return t->flags;
+}
+
+
 rasqal_expression*
 rasqal_new_1op_expression(rasqal_op op, rasqal_expression* arg)
 {
-  rasqal_expression* e=(rasqal_expression*)calloc(sizeof(rasqal_expression), 1);
+  rasqal_expression* e=(rasqal_expression*)RASQAL_CALLOC(rasqal_expression, sizeof(rasqal_expression), 1);
   e->op=op;
   e->arg1=arg;
   return e;
@@ -263,7 +315,7 @@ rasqal_new_2op_expression(rasqal_op op,
                           rasqal_expression* arg1, 
                           rasqal_expression* arg2)
 {
-  rasqal_expression* e=(rasqal_expression*)calloc(sizeof(rasqal_expression), 1);
+  rasqal_expression* e=(rasqal_expression*)RASQAL_CALLOC(rasqal_expression, sizeof(rasqal_expression), 1);
   e->op=op;
   e->arg1=arg1;
   e->arg2=arg2;
@@ -275,7 +327,7 @@ rasqal_new_string_op_expression(rasqal_op op,
                                 rasqal_expression* arg1,
                                 rasqal_literal* literal)
 {
-  rasqal_expression* e=(rasqal_expression*)calloc(sizeof(rasqal_expression), 1);
+  rasqal_expression* e=(rasqal_expression*)RASQAL_CALLOC(rasqal_expression, sizeof(rasqal_expression), 1);
   e->op=op;
   e->arg1=arg1;
   e->literal=literal;
@@ -285,13 +337,24 @@ rasqal_new_string_op_expression(rasqal_op op,
 rasqal_expression*
 rasqal_new_literal_expression(rasqal_literal *literal)
 {
-  rasqal_expression* e=(rasqal_expression*)calloc(sizeof(rasqal_expression), 1);
+  rasqal_expression* e=(rasqal_expression*)RASQAL_CALLOC(rasqal_expression, sizeof(rasqal_expression), 1);
   e->op=RASQAL_EXPR_LITERAL;
   e->literal=literal;
   return e;
 }
 
 
+rasqal_expression*
+rasqal_new_function_expression(raptor_uri* name,
+                               raptor_sequence* args) {
+  rasqal_expression* e=(rasqal_expression*)RASQAL_CALLOC(rasqal_expression, sizeof(rasqal_expression), 1);
+  e->op=RASQAL_EXPR_FUNCTION;
+  e->name=name;
+  e->args=args;
+  return e;
+}
+
+
 void
 rasqal_free_expression(rasqal_expression* e)
 {
@@ -316,6 +379,14 @@ rasqal_free_expression(rasqal_expression* e)
       break;
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
+    case RASQAL_EXPR_UMINUS:
+    case RASQAL_EXPR_BOUND:
+    case RASQAL_EXPR_STR:
+    case RASQAL_EXPR_LANG:
+    case RASQAL_EXPR_DATATYPE:
+    case RASQAL_EXPR_ISURI:
+    case RASQAL_EXPR_ISBLANK:
+    case RASQAL_EXPR_ISLITERAL:
       rasqal_free_expression(e->arg1);
       break;
     case RASQAL_EXPR_STR_MATCH:
@@ -325,8 +396,11 @@ rasqal_free_expression(rasqal_expression* e)
     case RASQAL_EXPR_LITERAL:
       rasqal_free_literal(e->literal);
       break;
+    case RASQAL_EXPR_FUNCTION:
+      raptor_free_uri(e->name);
+      raptor_free_sequence(e->args);
     default:
-      abort();
+      RASQAL_FATAL2("Unknown operation %d", e->op);
   }
   RASQAL_FREE(rasqal_expression, e);
 }
@@ -359,16 +433,25 @@ rasqal_expression_foreach(rasqal_expression* e,
       break;
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
+    case RASQAL_EXPR_UMINUS:
+    case RASQAL_EXPR_BOUND:
+    case RASQAL_EXPR_STR:
+    case RASQAL_EXPR_LANG:
+    case RASQAL_EXPR_DATATYPE:
+    case RASQAL_EXPR_ISURI:
+    case RASQAL_EXPR_ISBLANK:
+    case RASQAL_EXPR_ISLITERAL:
       return fn(user_data, e) ||
         rasqal_expression_foreach(e->arg1, fn, user_data);
       break;
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
     case RASQAL_EXPR_LITERAL:
+    case RASQAL_EXPR_FUNCTION:
       return fn(user_data, e);
       break;
     default:
-      abort();
+      RASQAL_FATAL2("Unknown operation %d", e->op);
   }
 }
 
@@ -413,6 +496,13 @@ rasqal_literal*
 rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
 {
   int error=0;
+  rasqal_literal* result=NULL;
+  
+#ifdef RASQAL_DEBUG
+  RASQAL_DEBUG2("evaluating expression %p: ", e);
+  rasqal_expression_print(e, stderr);
+  fprintf(stderr, "\n");
+#endif
   
   switch(e->op) {
     case RASQAL_EXPR_AND:
@@ -422,22 +512,23 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l=rasqal_expression_evaluate(query, e->arg1);
         if(!l)
-          return NULL;
+          goto failed;
         b=rasqal_literal_as_boolean(l, &error);
         rasqal_free_literal(l);
         if(error)
-          return NULL;
+          goto failed;
 
         if(b) {
           l=rasqal_expression_evaluate(query, e->arg2);
           if(!l)
-            return NULL;
+            goto failed;
           b=rasqal_literal_as_boolean(l, &error);
           rasqal_free_literal(l);
           if(error)
-            return NULL;
+            goto failed;
         }
-        return rasqal_new_boolean_literal(b);
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
       
     case RASQAL_EXPR_OR:
@@ -447,22 +538,23 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l=rasqal_expression_evaluate(query, e->arg1);
         if(!l)
-          return NULL;
+          goto failed;
         b=rasqal_literal_as_boolean(l, &error);
         rasqal_free_literal(l);
         if(error)
-          return NULL;
+          goto failed;
 
         if(!b) {
           l=rasqal_expression_evaluate(query, e->arg2);
           if(!l)
-            return NULL;
+            goto failed;
           b=rasqal_literal_as_boolean(l, &error);
           rasqal_free_literal(l);
           if(error)
-            return NULL;
+            goto failed;
         }
-        return rasqal_new_boolean_literal(b);
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_EQ:
@@ -472,20 +564,21 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, 0, &error) == 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_NEQ:
@@ -495,20 +588,21 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, 0, &error) != 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_LT:
@@ -518,20 +612,21 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, 0, &error) > 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_GT:
@@ -541,20 +636,21 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, 0, &error) < 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_LE:
@@ -564,20 +660,21 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, 0, &error) >= 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+        result=rasqal_new_boolean_literal(b);
+        break;        
       }
 
     case RASQAL_EXPR_GE:
@@ -587,124 +684,343 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, 0, &error) <= 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
+    case RASQAL_EXPR_UMINUS:
+      {
+        rasqal_literal *l=rasqal_expression_evaluate(query, e->arg1);
+        double d= -rasqal_literal_as_floating(l, &error);
+        rasqal_free_literal(l);
+        if(error)
+          goto failed;
+        result=rasqal_new_floating_literal(d);
+        break;
+      }
+
+    case RASQAL_EXPR_BOUND:
+      {
+        rasqal_literal *l;
+        rasqal_variable* v;
+        int b;
+
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+        
+        v=rasqal_literal_as_variable(l);
+        if(!v) {
+          rasqal_free_literal(l);
+          goto failed;
+        }
+
+        b=(v->value != NULL);
+
+        rasqal_free_literal(l);
+        result=rasqal_new_boolean_literal(b);
+        break;
+      }
+
+    case RASQAL_EXPR_STR:
+      {
+        rasqal_literal* l;
+        const unsigned char* s;
+        
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+
+        s=rasqal_literal_as_string(l);
+        if(!s) {
+          rasqal_free_literal(l);
+          goto failed;
+        }
+        
+        result=rasqal_new_string_literal(s, NULL, NULL, NULL);
+        rasqal_free_literal(l);
+
+        break;
+      }
+      
+    case RASQAL_EXPR_LANG:
+      {
+        rasqal_literal* l;
+        rasqal_variable* v;
+        unsigned char* new_language;
+        int free_literal=1;
+        
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+
+        v=rasqal_literal_as_variable(l);
+        if(v) {
+          rasqal_free_literal(l);
+          l=v->value;
+          free_literal=0;
+        }
+
+        if(l->type != RASQAL_LITERAL_STRING) {
+          if(free_literal)
+            rasqal_free_literal(l);
+          goto failed;
+        }
+        
+        if(l->language) {
+          new_language=(unsigned char*)RASQAL_MALLOC(cstring, strlen(l->language)+1);
+          strcpy((char*)new_language, l->language);
+        } else {
+          new_language=(unsigned char*)RASQAL_MALLOC(cstring, 2);
+          new_language[0]='-';
+          new_language[1]='\0';
+        }
+        
+        result=rasqal_new_string_literal(new_language, NULL, NULL, NULL);
+
+        if(free_literal)
+          rasqal_free_literal(l);
+
+        break;
+      }
+
+    case RASQAL_EXPR_DATATYPE:
+      {
+        rasqal_literal* l;
+        rasqal_variable* v;
+        int free_literal=1;
+        
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+
+        v=rasqal_literal_as_variable(l);
+        if(v) {
+          rasqal_free_literal(l);
+          l=v->value;
+          free_literal=0;
+        }
+
+        if(l->type != RASQAL_LITERAL_STRING) {
+          if(free_literal)
+            rasqal_free_literal(l);
+          goto failed;
+        }
+        
+        if(l->datatype)
+          result=rasqal_new_uri_literal(raptor_uri_copy(l->datatype));
+        else
+          result=NULL;
+
+        if(free_literal)
+          rasqal_free_literal(l);
+
+        break;
+      }
+
+    case RASQAL_EXPR_ISURI:
+      {
+        rasqal_literal *l;
+        rasqal_variable* v;
+        int free_literal=1;
+        int b;
+        
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+        
+        v=rasqal_literal_as_variable(l);
+        if(v) {
+          rasqal_free_literal(l);
+          l=v->value;
+          free_literal=0;
+        }
+
+        b=(l->type == RASQAL_LITERAL_URI);
+        
+        if(free_literal)
+          rasqal_free_literal(l);
+
+        result=rasqal_new_boolean_literal(b);
+        break;
+      }
+
+    case RASQAL_EXPR_ISBLANK:
+      {
+        rasqal_literal *l;
+        rasqal_variable* v;
+        int free_literal=1;
+        int b;
+        
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+        
+        v=rasqal_literal_as_variable(l);
+        if(v) {
+          rasqal_free_literal(l);
+          l=v->value;
+          free_literal=0;
+        }
+
+        b=(l->type == RASQAL_LITERAL_BLANK);
+
+        if(free_literal)
+          rasqal_free_literal(l);
+
+        result=rasqal_new_boolean_literal(b);
+        break;
+      }
+
+    case RASQAL_EXPR_ISLITERAL:
+      {
+        rasqal_literal *l;
+        rasqal_variable* v;
+        int free_literal=1;
+        int b;
+        
+        l=rasqal_expression_evaluate(query, e->arg1);
+        if(!l)
+          goto failed;
+        
+        v=rasqal_literal_as_variable(l);
+        if(v) {
+          rasqal_free_literal(l);
+          l=v->value;
+          free_literal=0;
+        }
+
+        b=(l->type == RASQAL_LITERAL_STRING);
+
+        if(free_literal)
+          rasqal_free_literal(l);
+
+        result=rasqal_new_boolean_literal(b);
+        break;
+      }
+      
     case RASQAL_EXPR_PLUS:
       {
         rasqal_literal *l1, *l2;
-        int i;
+        double d;
         int error1=0;
         int error2=0;
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
-        i=rasqal_literal_as_integer(l1, &error1) + 
-          rasqal_literal_as_integer(l2, &error2);
+        d=rasqal_literal_as_floating(l1, &error1) + 
+          rasqal_literal_as_floating(l2, &error2);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error || error1 || error2)
-          return NULL;
-        return rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+          goto failed;
+
+        result=rasqal_new_floating_literal(d);
+        break;
       }
       
     case RASQAL_EXPR_MINUS:
       {
         rasqal_literal *l1, *l2;
-        int i;
+        double d;
         int error1=0;
         int error2=0;
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
-        i=rasqal_literal_as_integer(l1, &error1) -
-          rasqal_literal_as_integer(l2, &error2);
+        d=rasqal_literal_as_floating(l1, &error1) -
+          rasqal_literal_as_floating(l2, &error2);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error || error1 || error2)
-          return NULL;
-        return rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+          goto failed;
+
+        result=rasqal_new_floating_literal(d);
+        break;
       }
       
     case RASQAL_EXPR_STAR:
       {
         rasqal_literal *l1, *l2;
-        int i;
+        double d;
         int error1=0;
         int error2=0;
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
-        i=rasqal_literal_as_integer(l1, &error1) *
-          rasqal_literal_as_integer(l2, &error2);
+        d=rasqal_literal_as_floating(l1, &error1) *
+          rasqal_literal_as_floating(l2, &error2);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error || error1 || error2)
-          return NULL;
-        return rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+          goto failed;
+
+        result=rasqal_new_floating_literal(d);
+        break;
       }
       
     case RASQAL_EXPR_SLASH:
       {
         rasqal_literal *l1, *l2;
-        int i;
+        double d;
         int error1=0;
         int error2=0;
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
-        i=rasqal_literal_as_integer(l1, &error1) /
-          rasqal_literal_as_integer(l2, &error2);
+        d=rasqal_literal_as_floating(l1, &error1) /
+          rasqal_literal_as_floating(l2, &error2);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error || error1 || error2)
-          return NULL;
-        return rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+          goto failed;
+        result=rasqal_new_floating_literal(d);
+        break;
       }
       
     case RASQAL_EXPR_REM:
@@ -716,12 +1032,12 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         i=rasqal_literal_as_integer(l1, &error1) %
@@ -729,8 +1045,10 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error || error1 || error2)
-          return NULL;
-        return rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+          goto failed;
+
+        result=rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+        break;
       }
       
     case RASQAL_EXPR_STR_EQ:
@@ -740,20 +1058,22 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, RASQAL_COMPARE_NOCASE, &error) == 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
       
     case RASQAL_EXPR_STR_NEQ:
@@ -763,20 +1083,22 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
 
         l2=rasqal_expression_evaluate(query, e->arg2);
         if(!l2) {
           rasqal_free_literal(l1);
-          return NULL;
+          goto failed;
         }
 
         b=(rasqal_literal_compare(l1, l2, RASQAL_COMPARE_NOCASE, &error) != 0);
         rasqal_free_literal(l1);
         rasqal_free_literal(l2);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_TILDE:
@@ -785,8 +1107,10 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         int i= ~ rasqal_literal_as_integer(l, &error);
         rasqal_free_literal(l);
         if(error)
-          return NULL;
-        return rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+          goto failed;
+
+        result=rasqal_new_integer_literal(RASQAL_LITERAL_INTEGER, i);
+        break;
       }
 
     case RASQAL_EXPR_BANG:
@@ -795,8 +1119,10 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         int b= ! rasqal_literal_as_boolean(l, &error);
         rasqal_free_literal(l);
         if(error)
-          return NULL;
-        return rasqal_new_boolean_literal(b);
+          goto failed;
+
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_STR_MATCH:
@@ -822,10 +1148,12 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         
         l1=rasqal_expression_evaluate(query, e->arg1);
         if(!l1)
-          return NULL;
+          goto failed;
         match_string=rasqal_literal_as_string(l1);
-        if(!match_string)
-          return NULL;
+        if(!match_string) {
+          rasqal_free_literal(l1);
+          goto failed;
+        }
         
         l2=e->literal;
         pattern=l2->string;
@@ -896,21 +1224,39 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         if(e->op == RASQAL_EXPR_STR_NMATCH)
           b=1-b;
 
+        rasqal_free_literal(l1);
+
         if(rc<0)
-          return NULL;
+          goto failed;
         
-        return rasqal_new_boolean_literal(b);
+        result=rasqal_new_boolean_literal(b);
+        break;
       }
 
     case RASQAL_EXPR_LITERAL:
-      return rasqal_new_literal_from_literal(e->literal);
+      result=rasqal_new_literal_from_literal(e->literal);
       break;
 
+    case RASQAL_EXPR_FUNCTION:
+      RASQAL_FATAL1("No function expressions yet");
+      break;
+      
     default:
-      abort();
+      RASQAL_FATAL2("Unknown operation %d", e->op);
   }
 
-  return NULL;
+  failed:
+
+#ifdef RASQAL_DEBUG
+  RASQAL_DEBUG2("result of %p: ", e);
+  if(result)
+    rasqal_literal_print(result, stderr);
+  else
+    fputs("(NULL)",stderr);
+  fputc('\n', stderr);
+#endif
+  
+  return result;
 }
 
 
@@ -924,6 +1270,7 @@ static const char* rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "gt",
   "le",
   "ge",
+  "uminus",
   "plus",
   "minus",
   "star",
@@ -937,6 +1284,14 @@ static const char* rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "bang",
   "literal",
   "pattern",
+  "function",
+  "bound",
+  "str",
+  "lang",
+  "datatype",
+  "isUri",
+  "isBlank",
+  "isLiteral"
 };
 
 void
@@ -989,6 +1344,14 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
       break;
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
+    case RASQAL_EXPR_UMINUS:
+    case RASQAL_EXPR_BOUND:
+    case RASQAL_EXPR_STR:
+    case RASQAL_EXPR_LANG:
+    case RASQAL_EXPR_DATATYPE:
+    case RASQAL_EXPR_ISURI:
+    case RASQAL_EXPR_ISBLANK:
+    case RASQAL_EXPR_ISLITERAL:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
@@ -1002,7 +1365,7 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
       fprintf(fh, "expr_pattern(%s)", (char*)e->value);
       break;
     default:
-      abort();
+      RASQAL_FATAL2("Unknown operation %d", e->op);
   }
   fputc(')', fh);
 }
