@@ -2,9 +2,9 @@
  *
  * rasqal_raptor.c - Rasqal triple store implementation with raptor
  *
- * $Id: rasqal_raptor.c,v 1.29 2004/10/29 16:19:36 cmdjb Exp $
+ * $Id: rasqal_raptor.c,v 1.36 2005/01/15 01:09:31 cmdjb Exp $
  * 
- * Copyright (C) 2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -86,7 +86,7 @@ raptor_uri* ordinal_as_uri(int ordinal)
   raptor_uri* uri;
   
   len=raptor_rdf_namespace_uri_len + 1; /* 1 for '_' */
-  while(t/10)
+  while(t/=10)
     len++;
   buffer=(unsigned char*)RASQAL_MALLOC(cstring, len+1);
   if(!buffer)
@@ -357,13 +357,14 @@ typedef struct {
 } rasqal_raptor_triples_match_context;
 
 
-static int
+static rasqal_triple_parts
 rasqal_raptor_bind_match(struct rasqal_triples_match_s* rtm,
                          void *user_data,
-                         rasqal_variable* bindings[4]) 
-{
+                         rasqal_variable* bindings[4],
+                         rasqal_triple_parts parts) {
   rasqal_raptor_triples_match_context* rtmc=(rasqal_raptor_triples_match_context*)rtm->user_data;
   int error=0;
+  rasqal_triple_parts result=(rasqal_triple_parts)0;
   
 #ifdef RASQAL_DEBUG
   if(rtmc->cur) {
@@ -376,35 +377,39 @@ rasqal_raptor_bind_match(struct rasqal_triples_match_s* rtm,
 
   /* set variable values from the fields of statement */
 
-  if(bindings[0]) {
+  if(bindings[0] && (parts & RASQAL_TRIPLE_SUBJECT)) {
     RASQAL_DEBUG1("binding subject to variable\n");
-    rasqal_variable_set_value(bindings[0], rasqal_literal_as_node(rtmc->cur->triple->subject));
+    rasqal_variable_set_value(bindings[0],
+                              rasqal_literal_as_node(rtmc->cur->triple->subject));
+    result= RASQAL_TRIPLE_SUBJECT;
   }
 
-  if(bindings[1]) {
+  if(bindings[1] && (parts & RASQAL_TRIPLE_PREDICATE)) {
     if(bindings[0] == bindings[1]) {
       if(rasqal_literal_compare(rtmc->cur->triple->subject,
                                 rtmc->cur->triple->predicate, 0, &error))
-        return 1;
+        return (rasqal_triple_parts)0;
       if(error)
-        return 1;
+        return (rasqal_triple_parts)0;
       
       RASQAL_DEBUG1("subject and predicate values match\n");
     } else {
       RASQAL_DEBUG1("binding predicate to variable\n");
-      rasqal_variable_set_value(bindings[1], rasqal_literal_as_node(rtmc->cur->triple->predicate));
+      rasqal_variable_set_value(bindings[1],
+                                rasqal_literal_as_node(rtmc->cur->triple->predicate));
+      result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_PREDICATE);
     }
   }
 
-  if(bindings[2]) {
+  if(bindings[2] && (parts & RASQAL_TRIPLE_OBJECT)) {
     int bind=1;
     
     if(bindings[0] == bindings[2]) {
       if(rasqal_literal_compare(rtmc->cur->triple->subject,
                                 rtmc->cur->triple->object, 0, &error))
-        return 1;
+        return (rasqal_triple_parts)0;
       if(error)
-        return 1;
+        return (rasqal_triple_parts)0;
 
       bind=0;
       RASQAL_DEBUG1("subject and object values match\n");
@@ -414,9 +419,9 @@ rasqal_raptor_bind_match(struct rasqal_triples_match_s* rtm,
        ) {
       if(rasqal_literal_compare(rtmc->cur->triple->predicate,
                                 rtmc->cur->triple->object, 0, &error))
-        return 1;
+        return (rasqal_triple_parts)0;
       if(error)
-        return 1;
+        return (rasqal_triple_parts)0;
 
       bind=0;
       RASQAL_DEBUG1("predicate and object values match\n");
@@ -424,17 +429,20 @@ rasqal_raptor_bind_match(struct rasqal_triples_match_s* rtm,
     
     if(bind) {
       RASQAL_DEBUG1("binding object to variable\n");
-      rasqal_variable_set_value(bindings[2],  rasqal_literal_as_node(rtmc->cur->triple->object));
+      rasqal_variable_set_value(bindings[2],
+                                rasqal_literal_as_node(rtmc->cur->triple->object));
+      result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_OBJECT);
     }
   }
 
-  if(bindings[3]) {
+  if(bindings[3] && (parts & RASQAL_TRIPLE_ORIGIN)) {
     rasqal_literal *l=rasqal_new_literal_from_literal(rtmc->cur->triple->origin);
     RASQAL_DEBUG1("binding origin to variable\n");
     rasqal_variable_set_value(bindings[3], l);
+    result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_ORIGIN);
   }
 
-  return 0;
+  return result;
 }
 
 
@@ -466,6 +474,18 @@ rasqal_raptor_finish_triples_match(struct rasqal_triples_match_s* rtm,
                                     void *user_data) {
   rasqal_raptor_triples_match_context* rtmc=(rasqal_raptor_triples_match_context*)rtm->user_data;
 
+  if(rtmc->match.subject)
+    rasqal_free_literal(rtmc->match.subject);
+
+  if(rtmc->match.predicate)
+    rasqal_free_literal(rtmc->match.predicate);
+
+  if(rtmc->match.object)
+    rasqal_free_literal(rtmc->match.object);
+
+  if(rtmc->match.origin)
+    rasqal_free_literal(rtmc->match.origin);
+
   RASQAL_FREE(rasqal_raptor_triples_match_context, rtmc);
 }
 
