@@ -2,9 +2,9 @@
  *
  * rdfproc.c - Redland RDF processor
  *
- * $Id: rdfproc.c,v 1.50 2004/11/01 20:43:35 cmdjb Exp $
+ * $Id: rdfproc.c,v 1.55 2005/02/03 22:58:54 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -86,7 +86,8 @@ enum command_type {
   CMD_SERIALIZE,
   CMD_REMOVE_CONTEXT,
   CMD_CONTEXTS,
-  CMD_MATCH
+  CMD_MATCH,
+  CMD_SIZE
 };
 
 typedef struct
@@ -124,6 +125,7 @@ static command commands[]={
   {CMD_REMOVE_CONTEXT, "remove-context", 1, 1, 0},
   {CMD_CONTEXTS, "contexts", 0, 0, 0},
   {CMD_MATCH, "match", 3, 4, 0},
+  {CMD_SIZE, "size", 0, 0, 0},
   {(enum command_type)-1, NULL}  
 };
  
@@ -200,7 +202,6 @@ main(int argc, char *argv[])
   librdf_serializer* serializer;
   librdf_storage *storage;
   librdf_model* model;
-  librdf_statement* partial_statement;
   librdf_node *source, *arc, *target, *node;
   librdf_node* context_node=NULL;
   librdf_stream* stream;
@@ -227,13 +228,15 @@ main(int argc, char *argv[])
   char *storage_name=(char*)default_storage_name;
   char *storage_options=(char*)default_storage_options;
   char *storage_password=NULL;
-  unsigned char *uri_string;
+  librdf_statement* partial_statement=NULL;
+  unsigned char *uri_string=NULL;
   int free_uri_string=0;
   librdf_model* output_model=NULL;
   librdf_storage* output_storage=NULL;
   librdf_serializer* output_serializer=NULL;
   size_t capacity;
   size_t size;
+  const char *query_graph_serializer_syntax_name="rdfxml";
 
   program=argv[0];
   if((p=strrchr(program, '/')))
@@ -488,6 +491,7 @@ main(int argc, char *argv[])
     puts("  source | target | arc NODE1 NODE2         Show 1 matching node.");
     puts("  arcs-in | arcs-out NODE                   Show properties in/out of NODE");
     puts("  has-arc-in | has-arc-out NODE ARC         Check for property in/out of NODE.");
+    puts("  size                                      Print the number of triples in the graph.");
     puts("\nNotation:");
     puts("  nodes are either blank node identifiers like _:ABC,");
     puts("    URIs like http://example.org otherwise are literal strings.");
@@ -744,7 +748,7 @@ main(int argc, char *argv[])
         }
       }
 
-      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2]);
+      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2], NULL);
       goto printmatching;
       break;
 
@@ -766,7 +770,7 @@ main(int argc, char *argv[])
         }
       }
 
-      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2]);
+      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2], NULL);
       if(!query) {
         fprintf(stderr, "%s: Failed to create new query %s\n", program, argv[2]);
         if(uri)
@@ -784,28 +788,73 @@ main(int argc, char *argv[])
         break;
       }
 
-      while(!librdf_query_results_finished(results)) {
-        fputs("result: [", stdout);
-        for(i=0; i<librdf_query_results_get_bindings_count(results); i++) {
-          librdf_node *value=librdf_query_results_get_binding_value(results, i);
-          name=(char*)librdf_query_results_get_binding_name(results, i);
-
-          if(i>0)
-            fputs(", ", stdout);
-          fprintf(stdout, "%s=", name);
-          if(value) {
-            librdf_node_print(value, stdout);
-            librdf_free_node(value);
-          } else
-            fputs("NULL", stdout);
-        }
-        fputs("]\n", stdout);
+      if (librdf_query_results_is_bindings(results)) {
+        fprintf(stderr, "%s: Query returned bindings results:\n", program);
         
-        librdf_query_results_next(results);
-      }
+        while(!librdf_query_results_finished(results)) {
+          fputs("result: [", stdout);
+          for(i=0; i<librdf_query_results_get_bindings_count(results); i++) {
+            librdf_node *value=librdf_query_results_get_binding_value(results, i);
+            name=(char*)librdf_query_results_get_binding_name(results, i);
 
-      fprintf(stdout, "%s: Query returned %d results\n", program, 
-              librdf_query_results_get_count(results));
+            if(i>0)
+              fputs(", ", stdout);
+            fprintf(stdout, "%s=", name);
+            if(value) {
+              librdf_node_print(value, stdout);
+              librdf_free_node(value);
+            } else
+              fputs("NULL", stdout);
+          }
+          fputs("]\n", stdout);
+
+          librdf_query_results_next(results);
+        }
+
+        fprintf(stdout, "%s: Query returned %d results\n", program, 
+                librdf_query_results_get_count(results));
+     } else if (librdf_query_results_is_boolean(results)) {
+       fprintf(stdout, "%s: Query returned boolean result: %s\n",
+               program,
+               librdf_query_results_get_boolean(results) ? "true" : "false");
+     } else if (librdf_query_results_is_graph(results)) {
+       librdf_storage* tmp_storage;
+       librdf_model* tmp_model;
+
+       tmp_storage=librdf_new_storage(world, NULL, NULL, NULL);
+       tmp_model=librdf_new_model(world, tmp_storage, NULL);
+
+       fprintf(stdout, "%s: Query returned graph result:\n", program);
+       
+       stream=librdf_query_results_as_stream(results);
+       if(!stream) {
+         fprintf(stderr, "%s: Failed to get query results graph\n", program);
+         return(1);
+       }
+       librdf_model_add_statements(tmp_model, stream);
+       librdf_free_stream(stream);
+
+       fprintf(stdout, "%s: Total %d triples\n", program, librdf_model_size(model));
+
+       serializer=librdf_new_serializer(world, 
+                                        query_graph_serializer_syntax_name, 
+                                        NULL, NULL);
+       if(!serializer) {
+         fprintf(stderr, 
+                 "%s: Failed to create serializer type %s\n", program,
+                 query_graph_serializer_syntax_name);
+         return(1);
+       }
+
+       librdf_serializer_serialize_model_to_file_handle(serializer, stdout, NULL, tmp_model);
+       librdf_free_serializer(serializer);
+       librdf_free_model(tmp_model);
+       librdf_free_storage(tmp_storage);
+     } else {
+       fprintf(stdout, "%s: Query returned unknown result format\n", program);
+       rc=1;
+     }
+      
       if(uri)
         librdf_free_uri(uri);
       librdf_free_query_results(results);
@@ -1257,6 +1306,14 @@ main(int argc, char *argv[])
       break;
 
 
+    case CMD_SIZE:
+      count=librdf_model_size(model);
+      if(count >= 0)
+        fprintf(stderr, "%s: graph has %d triples\n", program, count);
+      else
+        fprintf(stderr, "%s: graph has unknown number of triples\n", program);
+      break;
+
     default:
       fprintf(stderr, "%s: Unknown command %d\n", program, type);
       return(1);
